@@ -15,25 +15,21 @@
 #    under the License.
 
 import functools
-import math
-import time
 
 import microversion_parse
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
 from oslo_utils import strutils
 import six
 import webob
 
 from nova.api.openstack import api_version_request as api_version
 from nova.api.openstack import versioned_method
+from nova.api import wsgi
 from nova import exception
 from nova import i18n
 from nova.i18n import _
-from nova.i18n import _LE
-from nova.i18n import _LI
-from nova import utils
-from nova import wsgi
 
 
 LOG = logging.getLogger(__name__)
@@ -43,11 +39,6 @@ _SUPPORTED_CONTENT_TYPES = (
     'application/vnd.openstack.compute+json',
 )
 
-_MEDIA_TYPE_MAP = {
-    'application/vnd.openstack.compute+json': 'json',
-    'application/json': 'json',
-}
-
 # These are typically automatically created by routes as either defaults
 # collection or member methods.
 _ROUTES_METHODS = [
@@ -83,94 +74,14 @@ def get_supported_content_types():
     return _SUPPORTED_CONTENT_TYPES
 
 
-def get_media_map():
-    return dict(_MEDIA_TYPE_MAP.items())
-
-
-# NOTE(rlrossit): This function allows a get on both a dict-like and an
-# object-like object. cache_db_items() is used on both versioned objects and
-# dicts, so the function can't be totally changed over to [] syntax, nor
-# can it be changed over to use getattr().
-def item_get(item, item_key):
-    if hasattr(item, '__getitem__'):
-        return item[item_key]
-    else:
-        return getattr(item, item_key)
-
-
 class Request(wsgi.Request):
     """Add some OpenStack API-specific logic to the base webob.Request."""
 
     def __init__(self, *args, **kwargs):
         super(Request, self).__init__(*args, **kwargs)
-        self._extension_data = {'db_items': {}}
         if not hasattr(self, 'api_version_request'):
             self.api_version_request = api_version.APIVersionRequest()
 
-    def cache_db_items(self, key, items, item_key='id'):
-        """Allow API methods to store objects from a DB query to be
-        used by API extensions within the same API request.
-
-        An instance of this class only lives for the lifetime of a
-        single API request, so there's no need to implement full
-        cache management.
-        """
-        db_items = self._extension_data['db_items'].setdefault(key, {})
-        for item in items:
-            db_items[item_get(item, item_key)] = item
-
-    def get_db_items(self, key):
-        """Allow an API extension to get previously stored objects within
-        the same API request.
-
-        Note that the object data will be slightly stale.
-        """
-        return self._extension_data['db_items'][key]
-
-    def get_db_item(self, key, item_key):
-        """Allow an API extension to get a previously stored object
-        within the same API request.
-
-        Note that the object data will be slightly stale.
-        """
-        return self.get_db_items(key).get(item_key)
-
-    def cache_db_instances(self, instances):
-        self.cache_db_items('instances', instances, 'uuid')
-
-    def cache_db_instance(self, instance):
-        self.cache_db_items('instances', [instance], 'uuid')
-
-    def get_db_instances(self):
-        return self.get_db_items('instances')
-
-    def get_db_instance(self, instance_uuid):
-        return self.get_db_item('instances', instance_uuid)
-
-    def cache_db_flavors(self, flavors):
-        self.cache_db_items('flavors', flavors, 'flavorid')
-
-    def cache_db_flavor(self, flavor):
-        self.cache_db_items('flavors', [flavor], 'flavorid')
-
-    def get_db_flavors(self):
-        return self.get_db_items('flavors')
-
-    def get_db_flavor(self, flavorid):
-        return self.get_db_item('flavors', flavorid)
-
-    def cache_db_compute_nodes(self, compute_nodes):
-        self.cache_db_items('compute_nodes', compute_nodes, 'id')
-
-    def cache_db_compute_node(self, compute_node):
-        self.cache_db_items('compute_nodes', [compute_node], 'id')
-
-    def get_db_compute_nodes(self):
-        return self.get_db_items('compute_nodes')
-
-    def get_db_compute_node(self, id):
-        return self.get_db_item('compute_nodes', id)
-
     def best_match_content_type(self):
         """Determine the requested response content-type."""
         if 'nova.best_content_type' not in self.environ:
@@ -185,8 +96,10 @@ class Request(wsgi.Request):
                     content_type = possible_type
 
             if not content_type:
-                content_type = self.accept.best_match(
+                best_matches = self.accept.acceptable_offers(
                     get_supported_content_types())
+                if best_matches:
+                    content_type = best_matches[0][0]
 
             self.environ['nova.best_content_type'] = (content_type or
                                                       'application/json')
@@ -223,8 +136,19 @@ class Request(wsgi.Request):
         """
         if not self.accept_language:
             return None
-        return self.accept_language.best_match(
-                i18n.get_available_languages())
+
+        # NOTE(takashin): To decide the default behavior, 'default' is
+        # preferred over 'default_tag' because that is return as it is when
+        # no match. This is also little tricky that 'default' value cannot be
+        # None. At least one of default_tag or default must be supplied as
+        # an argument to the method, to define the defaulting behavior.
+        # So passing a sentinal value to return None from this function.
+        best_match = self.accept_language.lookup(
+            i18n.get_available_languages(), default='fake_LANG')
+
+        if best_match == 'fake_LANG':
+            best_match = None
+        return best_match
 
     def set_api_version_request(self):
         """Set API version request based on the request header information."""
@@ -350,6 +274,9 @@ class ResponseObject(object):
 
         Utility method for serializing the wrapped object.  Returns a
         webob.Response object.
+
+        Header values are set to the appropriate Python type and
+        encoding demanded by PEP 3333: whatever the native str type is.
         """
 
         serializer = self.serializer
@@ -358,16 +285,19 @@ class ResponseObject(object):
         if self.obj is not None:
             body = serializer.serialize(self.obj)
         response = webob.Response(body=body)
-        if response.headers.get('Content-Length'):
-            # NOTE(andreykurilin): we need to encode 'Content-Length' header,
-            # since webob.Response auto sets it if "body" attr is presented.
-            # https://github.com/Pylons/webob/blob/1.5.0b0/webob/response.py#L147
-            response.headers['Content-Length'] = utils.utf8(
-                response.headers['Content-Length'])
         response.status_int = self.code
-        for hdr, value in self._headers.items():
-            response.headers[hdr] = utils.utf8(value)
-        response.headers['Content-Type'] = utils.utf8(content_type)
+        for hdr, val in self._headers.items():
+            # In Py3.X Headers must be a str that was first safely
+            # encoded to UTF-8 (to catch any bad encodings) and then
+            # decoded back to a native str.
+            response.headers[hdr] = encodeutils.safe_decode(
+                    encodeutils.safe_encode(val))
+        # Deal with content_type
+        if not isinstance(content_type, six.text_type):
+            content_type = six.text_type(content_type)
+        # In Py3.X Headers must be a str.
+        response.headers['Content-Type'] = encodeutils.safe_decode(
+                encodeutils.safe_encode(content_type))
         return response
 
     @property
@@ -409,7 +339,7 @@ class ResourceExceptionHandler(object):
     """Context manager to handle Resource exceptions.
 
     Used when processing exceptions generated by API implementation
-    methods (or their extensions).  Converts most exceptions to Fault
+    methods.  Converts most exceptions to Fault
     exceptions, with the appropriate logging.
     """
 
@@ -431,14 +361,14 @@ class ResourceExceptionHandler(object):
                     explanation=ex_value.format_message()))
         elif isinstance(ex_value, TypeError):
             exc_info = (ex_type, ex_value, ex_traceback)
-            LOG.error(_LE('Exception handling resource: %s'), ex_value,
+            LOG.error('Exception handling resource: %s', ex_value,
                       exc_info=exc_info)
             raise Fault(webob.exc.HTTPBadRequest())
         elif isinstance(ex_value, Fault):
-            LOG.info(_LI("Fault thrown: %s"), ex_value)
+            LOG.info("Fault thrown: %s", ex_value)
             raise ex_value
         elif isinstance(ex_value, webob.exc.HTTPException):
-            LOG.info(_LI("HTTP exception thrown: %s"), ex_value)
+            LOG.info("HTTP exception thrown: %s", ex_value)
             raise Fault(ex_value)
 
         # We didn't handle the exception
@@ -460,15 +390,11 @@ class Resource(wsgi.Application):
     wrapped in Fault() to provide API friendly error responses.
 
     """
-    support_api_request_version = False
+    support_api_request_version = True
 
-    def __init__(self, controller, inherits=None):
+    def __init__(self, controller):
         """:param controller: object that implement methods created by routes
                               lib
-           :param inherits: another resource object that this resource should
-                            inherit extensions from. Any action extensions that
-                            are applied to the parent resource will also apply
-                            to this resource.
         """
 
         self.controller = controller
@@ -480,11 +406,6 @@ class Resource(wsgi.Application):
         if controller:
             self.register_actions(controller)
 
-        # Save a mapping of extensions
-        self.wsgi_extensions = {}
-        self.wsgi_action_extensions = {}
-        self.inherits = inherits
-
     def register_actions(self, controller):
         """Registers controller actions with this resource."""
 
@@ -492,25 +413,6 @@ class Resource(wsgi.Application):
         for key, method_name in actions.items():
             self.wsgi_actions[key] = getattr(controller, method_name)
 
-    def register_extensions(self, controller):
-        """Registers controller extensions with this resource."""
-
-        extensions = getattr(controller, 'wsgi_extensions', [])
-        for method_name, action_name in extensions:
-            # Look up the extending method
-            extension = getattr(controller, method_name)
-
-            if action_name:
-                # Extending an action...
-                if action_name not in self.wsgi_action_extensions:
-                    self.wsgi_action_extensions[action_name] = []
-                self.wsgi_action_extensions[action_name].append(extension)
-            else:
-                # Extending a regular method
-                if method_name not in self.wsgi_extensions:
-                    self.wsgi_extensions[method_name] = []
-                self.wsgi_extensions[method_name].append(extension)
-
     def get_action_args(self, request_environment):
         """Parse dictionary created by routes library."""
 
@@ -544,30 +446,6 @@ class Resource(wsgi.Application):
     def deserialize(self, body):
         return JSONDeserializer().deserialize(body)
 
-    def process_extensions(self, extensions, resp_obj, request,
-                           action_args):
-        for ext in extensions:
-            response = None
-            # Regular functions get post-processing...
-            try:
-                with ResourceExceptionHandler():
-                    response = ext(req=request, resp_obj=resp_obj,
-                                   **action_args)
-            except exception.VersionNotFoundForAPIMethod:
-                # If an attached extension (@wsgi.extends) for the
-                # method has no version match its not an error. We
-                # just don't run the extends code
-                continue
-            except Fault as ex:
-                response = ex
-
-            # We had a response return it, to exit early. This is
-            # actually a failure mode. None is success.
-            if response:
-                return response
-
-        return None
-
     def _should_have_body(self, request):
         return request.method in _METHODS_WITH_BODY
 
@@ -614,8 +492,8 @@ class Resource(wsgi.Application):
 
         # Get the implementing method
         try:
-            meth, extensions = self.get_method(request, action,
-                                               content_type, body)
+            meth = self.get_method(request, action,
+                                   content_type, body)
         except (AttributeError, TypeError):
             return Fault(webob.exc.HTTPNotFound())
         except KeyError as ex:
@@ -637,13 +515,7 @@ class Resource(wsgi.Application):
 
         # Now, deserialize the request body...
         try:
-            contents = {}
-            if self._should_have_body(request):
-                # allow empty body with PUT and POST
-                if request.content_length == 0:
-                    contents = {'body': None}
-                else:
-                    contents = self.deserialize(body)
+            contents = self._get_request_content(body, request)
         except exception.MalformedRequestBody:
             msg = _("Malformed request body")
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
@@ -684,17 +556,17 @@ class Resource(wsgi.Application):
                 # Do a preserialize to set up the response object
                 if hasattr(meth, 'wsgi_code'):
                     resp_obj._default_code = meth.wsgi_code
-                # Process extensions
-                response = self.process_extensions(extensions, resp_obj,
-                                                        request, action_args)
 
             if resp_obj and not response:
                 response = resp_obj.serialize(request, accept)
 
         if hasattr(response, 'headers'):
             for hdr, val in list(response.headers.items()):
-                # Headers must be utf-8 strings
-                response.headers[hdr] = utils.utf8(val)
+                if not isinstance(val, six.text_type):
+                    val = six.text_type(val)
+                # In Py3.X Headers must be a string
+                response.headers[hdr] = encodeutils.safe_decode(
+                        encodeutils.safe_encode(val))
 
             if not request.api_version_request.is_null():
                 response.headers[API_VERSION_REQUEST_HEADER] = \
@@ -706,43 +578,44 @@ class Resource(wsgi.Application):
 
         return response
 
+    def _get_request_content(self, body, request):
+        contents = {}
+        if self._should_have_body(request):
+            # allow empty body with PUT and POST
+            if request.content_length == 0 or request.content_length is None:
+                contents = {'body': None}
+            else:
+                contents = self.deserialize(body)
+        return contents
+
     def get_method(self, request, action, content_type, body):
-        meth, extensions = self._get_method(request,
-                                            action,
-                                            content_type,
-                                            body)
-        if self.inherits:
-            _meth, parent_ext = self.inherits.get_method(request,
-                                                         action,
-                                                         content_type,
-                                                         body)
-            extensions.extend(parent_ext)
-        return meth, extensions
+        meth = self._get_method(request,
+                                action,
+                                content_type,
+                                body)
+        return meth
 
     def _get_method(self, request, action, content_type, body):
-        """Look up the action-specific method and its extensions."""
+        """Look up the action-specific method."""
         # Look up the method
         try:
             if not self.controller:
                 meth = getattr(self, action)
             else:
                 meth = getattr(self.controller, action)
+            return meth
         except AttributeError:
             if (not self.wsgi_actions or
                     action not in _ROUTES_METHODS + ['action']):
                 # Propagate the error
                 raise
-        else:
-            return meth, self.wsgi_extensions.get(action, [])
-
         if action == 'action':
             action_name = action_peek(body)
         else:
             action_name = action
 
         # Look up the action method
-        return (self.wsgi_actions[action_name],
-                self.wsgi_action_extensions.get(action_name, []))
+        return (self.wsgi_actions[action_name])
 
     def dispatch(self, method, request, action_args):
         """Dispatch a call to the action-specific method."""
@@ -756,10 +629,6 @@ class Resource(wsgi.Application):
             return Fault(webob.exc.HTTPNotFound())
 
 
-class ResourceV21(Resource):
-    support_api_request_version = True
-
-
 def action(name):
     """Mark a function as an action.
 
@@ -775,32 +644,53 @@ def action(name):
     return decorator
 
 
-def extends(*args, **kwargs):
-    """Indicate a function extends an operation.
+def expected_errors(errors):
+    """Decorator for v2.1 API methods which specifies expected exceptions.
 
-    Can be used as either::
-
-        @extends
-        def index(...):
-            pass
-
-    or as::
-
-        @extends(action='resize')
-        def _action_resize(...):
-            pass
+    Specify which exceptions may occur when an API method is called. If an
+    unexpected exception occurs then return a 500 instead and ask the user
+    of the API to file a bug report.
     """
+    def decorator(f):
+        @functools.wraps(f)
+        def wrapped(*args, **kwargs):
+            try:
+                return f(*args, **kwargs)
+            except Exception as exc:
+                if isinstance(exc, webob.exc.WSGIHTTPException):
+                    if isinstance(errors, int):
+                        t_errors = (errors,)
+                    else:
+                        t_errors = errors
+                    if exc.code in t_errors:
+                        raise
+                elif isinstance(exc, exception.Forbidden):
+                    # Note(cyeoh): Special case to handle
+                    # Forbidden exceptions so every
+                    # extension method does not need to wrap authorize
+                    # calls. ResourceExceptionHandler silently
+                    # converts NotAuthorized to HTTPForbidden
+                    raise
+                elif isinstance(exc, exception.ValidationError):
+                    # Note(oomichi): Handle a validation error, which
+                    # happens due to invalid API parameters, as an
+                    # expected error.
+                    raise
+                elif isinstance(exc, exception.Unauthorized):
+                    # Handle an authorized exception, will be
+                    # automatically converted to a HTTP 401, clients
+                    # like python-novaclient handle this error to
+                    # generate new token and do another attempt.
+                    raise
 
-    def decorator(func):
-        # Store enough information to find what we're extending
-        func.wsgi_extends = (func.__name__, kwargs.get('action'))
-        return func
+                LOG.exception("Unexpected exception in API method")
+                msg = _('Unexpected API Error. Please report this at '
+                    'http://bugs.launchpad.net/nova/ and attach the Nova '
+                    'API log if possible.\n%s') % type(exc)
+                raise webob.exc.HTTPInternalServerError(explanation=msg)
 
-    # If we have positional arguments, call the decorator
-    if args:
-        return decorator(*args)
+        return wrapped
 
-    # OK, return the decorator instead
     return decorator
 
 
@@ -816,7 +706,6 @@ class ControllerMetaclass(type):
 
         # Find all actions
         actions = {}
-        extensions = []
         versioned_methods = None
         # start with wsgi actions from base classes
         for base in bases:
@@ -838,12 +727,9 @@ class ControllerMetaclass(type):
                 continue
             if getattr(value, 'wsgi_action', None):
                 actions[value.wsgi_action] = key
-            elif getattr(value, 'wsgi_extends', None):
-                extensions.append(value.wsgi_extends)
 
-        # Add the actions and extensions to the class dict
+        # Add the actions to the class dict
         cls_dict['wsgi_actions'] = actions
-        cls_dict['wsgi_extensions'] = extensions
         if versioned_methods:
             cls_dict[VER_METHOD_ATTR] = versioned_methods
 
@@ -851,17 +737,14 @@ class ControllerMetaclass(type):
                                                        cls_dict)
 
 
-@six.add_metaclass(ControllerMetaclass)
-class Controller(object):
+class Controller(metaclass=ControllerMetaclass):
     """Default controller."""
 
     _view_builder_class = None
 
-    def __init__(self, view_builder=None):
+    def __init__(self):
         """Initialize controller with a view builder instance."""
-        if view_builder:
-            self._view_builder = view_builder
-        elif self._view_builder_class:
+        if self._view_builder_class:
             self._view_builder = self._view_builder_class()
         else:
             self._view_builder = None
@@ -1070,46 +953,3 @@ class Fault(webob.exc.HTTPException):
 
     def __str__(self):
         return self.wrapped_exc.__str__()
-
-
-class RateLimitFault(webob.exc.HTTPException):
-    """Rate-limited request response."""
-
-    def __init__(self, message, details, retry_time):
-        """Initialize new `RateLimitFault` with relevant information."""
-        hdrs = RateLimitFault._retry_after(retry_time)
-        self.wrapped_exc = webob.exc.HTTPTooManyRequests(headers=hdrs)
-        self.content = {
-            "overLimit": {
-                "code": self.wrapped_exc.status_int,
-                "message": message,
-                "details": details,
-                "retryAfter": hdrs['Retry-After'],
-            },
-        }
-
-    @staticmethod
-    def _retry_after(retry_time):
-        delay = int(math.ceil(retry_time - time.time()))
-        retry_after = delay if delay > 0 else 0
-        headers = {'Retry-After': '%d' % retry_after}
-        return headers
-
-    @webob.dec.wsgify(RequestClass=Request)
-    def __call__(self, request):
-        """Return the wrapped exception with a serialized body conforming
-        to our error format.
-        """
-        user_locale = request.best_match_language()
-
-        self.content['overLimit']['message'] = \
-            i18n.translate(self.content['overLimit']['message'], user_locale)
-        self.content['overLimit']['details'] = \
-            i18n.translate(self.content['overLimit']['details'], user_locale)
-
-        content = JSONDictSerializer().serialize(self.content)
-        self.wrapped_exc.charset = 'UTF-8'
-        self.wrapped_exc.content_type = "application/json"
-        self.wrapped_exc.text = content
-
-        return self.wrapped_exc
