@@ -14,6 +14,7 @@
 #    under the License.
 
 from oslo_config import cfg
+from oslo_config import types
 
 vnc_group = cfg.OptGroup(
     'vnc',
@@ -35,38 +36,26 @@ Guests will get created with graphical devices to support this. Clients
 (for example Horizon) can then establish a VNC connection to the guest.
 """),
 
-    cfg.StrOpt(
-        'keymap',
-        default='en-us',
-        deprecated_group='DEFAULT',
-        deprecated_name='vnc_keymap',
-        help="""
-Keymap for VNC.
-
-The keyboard mapping (keymap) determines which keyboard layout a VNC
-session should use by default.
-
-Possible values:
-
-* A keyboard layout which is supported by the underlying hypervisor on
-  this node. This is usually an 'IETF language tag' (for example
-  'en-us').  If you use QEMU as hypervisor, you should find the  list
-  of supported keyboard layouts at ``/usr/share/qemu/keymaps``.
-"""),
-
-    cfg.StrOpt(
-        'vncserver_listen',
+    cfg.HostAddressOpt(
+        'server_listen',
         default='127.0.0.1',
-        deprecated_group='DEFAULT',
+        deprecated_opts=[
+            cfg.DeprecatedOpt('vncserver_listen', group='DEFAULT'),
+            cfg.DeprecatedOpt('vncserver_listen', group='vnc'),
+        ],
         help="""
 The IP address or hostname on which an instance should listen to for
 incoming VNC connection requests on this node.
 """),
 
-    cfg.StrOpt(
-        'vncserver_proxyclient_address',
+    cfg.HostAddressOpt(
+        'server_proxyclient_address',
         default='127.0.0.1',
-        deprecated_group='DEFAULT',
+        deprecated_opts=[
+            cfg.DeprecatedOpt('vncserver_proxyclient_address',
+                              group='DEFAULT'),
+            cfg.DeprecatedOpt('vncserver_proxyclient_address', group='vnc'),
+        ],
         help="""
 Private, internal IP address or hostname of VNC console proxy.
 
@@ -74,7 +63,7 @@ The VNC proxy is an OpenStack component that enables compute service
 users to access their instances through VNC clients.
 
 This option sets the private address to which proxy clients, such as
-``nova-xvpvncproxy``, should connect to.
+``nova-novncproxy``, should connect to.
 """),
 
     cfg.URIOpt(
@@ -92,78 +81,15 @@ This option sets the public base URL to which client systems will
 connect. noVNC clients can use this address to connect to the noVNC
 instance and, by extension, the VNC sessions.
 
+If using noVNC >= 1.0.0, you should use ``vnc_lite.html`` instead of
+``vnc_auto.html``.
+
 Related options:
 
 * novncproxy_host
 * novncproxy_port
 """),
 
-    cfg.StrOpt(
-        'xvpvncproxy_host',
-        default='0.0.0.0',
-        deprecated_group='DEFAULT',
-        help="""
-IP address or hostname that the XVP VNC console proxy should bind to.
-
-The VNC proxy is an OpenStack component that enables compute service
-users to access their instances through VNC clients. Xen provides
-the Xenserver VNC Proxy, or XVP, as an alternative to the
-websocket-based noVNC proxy used by Libvirt. In contrast to noVNC,
-XVP clients are Java-based.
-
-This option sets the private address to which the XVP VNC console proxy
-service should bind to.
-
-Related options:
-
-* xvpvncproxy_port
-* xvpvncproxy_base_url
-"""),
-
-    cfg.PortOpt(
-        'xvpvncproxy_port',
-        default=6081,
-        deprecated_group='DEFAULT',
-        help="""
-Port that the XVP VNC console proxy should bind to.
-
-The VNC proxy is an OpenStack component that enables compute service
-users to access their instances through VNC clients. Xen provides
-the Xenserver VNC Proxy, or XVP, as an alternative to the
-websocket-based noVNC proxy used by Libvirt. In contrast to noVNC,
-XVP clients are Java-based.
-
-This option sets the private port to which the XVP VNC console proxy
-service should bind to.
-
-Related options:
-
-* xvpvncproxy_host
-* xvpvncproxy_base_url
-"""),
-
-    cfg.URIOpt(
-        'xvpvncproxy_base_url',
-        default='http://127.0.0.1:6081/console',
-        deprecated_group='DEFAULT',
-        help="""
-Public URL address of XVP VNC console proxy.
-
-The VNC proxy is an OpenStack component that enables compute service
-users to access their instances through VNC clients. Xen provides
-the Xenserver VNC Proxy, or XVP, as an alternative to the
-websocket-based noVNC proxy used by Libvirt. In contrast to noVNC,
-XVP clients are Java-based.
-
-This option sets the public base URL to which client systems will
-connect. XVP clients can use this address to connect to the XVP
-instance and, by extension, the VNC sessions.
-
-Related options:
-
-* xvpvncproxy_host
-* xvpvncproxy_port
-"""),
 ]
 
 CLI_OPTS = [
@@ -205,6 +131,60 @@ Related options:
 
 * novncproxy_host
 * novncproxy_base_url
+"""),
+    cfg.ListOpt(
+        'auth_schemes',
+        item_type=types.String(choices=(
+            ('none', 'Allow connection without authentication'),
+            ('vencrypt', 'Use VeNCrypt authentication scheme'),
+        )),
+        default=['none'],
+        help="""
+The authentication schemes to use with the compute node.
+
+Control what RFB authentication schemes are permitted for connections between
+the proxy and the compute host. If multiple schemes are enabled, the first
+matching scheme will be used, thus the strongest schemes should be listed
+first.
+
+Related options:
+
+* ``[vnc]vencrypt_client_key``, ``[vnc]vencrypt_client_cert``: must also be set
+"""),
+    cfg.StrOpt(
+        'vencrypt_client_key',
+        help="""The path to the client certificate PEM file (for x509)
+
+The fully qualified path to a PEM file containing the private key which the VNC
+proxy server presents to the compute node during VNC authentication.
+
+Related options:
+
+* ``vnc.auth_schemes``: must include ``vencrypt``
+* ``vnc.vencrypt_client_cert``: must also be set
+"""),
+    cfg.StrOpt(
+        'vencrypt_client_cert',
+        help="""The path to the client key file (for x509)
+
+The fully qualified path to a PEM file containing the x509 certificate which
+the VNC proxy server presents to the compute node during VNC authentication.
+
+Realted options:
+
+* ``vnc.auth_schemes``: must include ``vencrypt``
+* ``vnc.vencrypt_client_key``: must also be set
+"""),
+    cfg.StrOpt(
+        'vencrypt_ca_certs',
+        help="""The path to the CA certificate PEM file
+
+The fully qualified path to a PEM file containing one or more x509 certificates
+for the certificate authorities used by the compute node VNC server.
+
+Related options:
+
+* ``vnc.auth_schemes``: must include ``vencrypt``
 """),
 ]
 
