@@ -12,8 +12,10 @@
 
 
 from oslo_db.sqlalchemy import models
+from oslo_log import log as logging
 from sqlalchemy import Boolean
 from sqlalchemy import Column
+from sqlalchemy import DateTime
 from sqlalchemy.dialects.mysql import MEDIUMTEXT
 from sqlalchemy import Enum
 from sqlalchemy.ext.declarative import declarative_base
@@ -28,6 +30,8 @@ from sqlalchemy import String
 from sqlalchemy import Text
 from sqlalchemy import Unicode
 
+LOG = logging.getLogger(__name__)
+
 
 def MediumText():
     return Text().with_variant(MEDIUMTEXT(), 'mysql')
@@ -116,6 +120,7 @@ class CellMapping(API_BASE):
     name = Column(String(255))
     transport_url = Column(Text())
     database_connection = Column(Text())
+    disabled = Column(Boolean, default=False)
     host_mapping = orm.relationship('HostMapping',
                             backref=backref('cell_mapping', uselist=False),
                             foreign_keys=id,
@@ -129,13 +134,19 @@ class InstanceMapping(API_BASE):
     __table_args__ = (Index('project_id_idx', 'project_id'),
                       Index('instance_uuid_idx', 'instance_uuid'),
                       schema.UniqueConstraint('instance_uuid',
-                          name='uniq_instance_mappings0instance_uuid'))
+                          name='uniq_instance_mappings0instance_uuid'),
+                      Index('instance_mappings_user_id_project_id_idx',
+                            'user_id', 'project_id'))
 
     id = Column(Integer, primary_key=True)
     instance_uuid = Column(String(36), nullable=False)
     cell_id = Column(Integer, ForeignKey('cell_mappings.id'),
             nullable=True)
     project_id = Column(String(255), nullable=False)
+    # FIXME(melwitt): This should eventually be non-nullable, but we need a
+    # transition period first.
+    user_id = Column(String(255), nullable=True)
+    queued_for_delete = Column(Boolean)
     cell_mapping = orm.relationship('CellMapping',
             backref=backref('instance_mapping', uselist=False),
             foreign_keys=cell_id,
@@ -167,7 +178,7 @@ class RequestSpec(API_BASE):
 
     id = Column(Integer, primary_key=True)
     instance_uuid = Column(String(36), nullable=False)
-    spec = Column(Text, nullable=False)
+    spec = Column(MediumText(), nullable=False)
 
 
 class Flavors(API_BASE):
@@ -189,6 +200,7 @@ class Flavors(API_BASE):
     vcpu_weight = Column(Integer)
     disabled = Column(Boolean, default=False)
     is_public = Column(Boolean, default=True)
+    description = Column(Text)
 
 
 class FlavorExtraSpecs(API_BASE):
@@ -238,10 +250,12 @@ class BuildRequest(API_BASE):
         )
 
     id = Column(Integer, primary_key=True)
+    # TODO(mriedem): instance_uuid should be nullable=False
     instance_uuid = Column(String(36))
     project_id = Column(String(255), nullable=False)
-    instance = Column(Text)
+    instance = Column(MediumText())
     block_device_mappings = Column(MediumText())
+    tags = Column(Text())
     # TODO(alaski): Drop these from the db in Ocata
     # columns_to_drop = ['request_spec_id', 'user_id', 'display_name',
     #         'instance_metadata', 'progress', 'vm_state', 'task_state',
@@ -270,6 +284,19 @@ class KeyPair(API_BASE):
                   nullable=False, server_default='ssh')
 
 
+# TODO(stephenfin): Remove this as it's now unused post-placement split
+class ResourceClass(API_BASE):
+    """Represents the type of resource for an inventory or allocation."""
+    __tablename__ = 'resource_classes'
+    __table_args__ = (
+        schema.UniqueConstraint("name", name="uniq_resource_classes0name"),
+    )
+
+    id = Column(Integer, primary_key=True, nullable=False)
+    name = Column(String(255), nullable=False)
+
+
+# TODO(stephenfin): Remove this as it's now unused post-placement split
 class ResourceProvider(API_BASE):
     """Represents a mapping to a providers of resources."""
 
@@ -279,6 +306,10 @@ class ResourceProvider(API_BASE):
         schema.UniqueConstraint('uuid',
             name='uniq_resource_providers0uuid'),
         Index('resource_providers_name_idx', 'name'),
+        Index('resource_providers_root_provider_id_idx',
+              'root_provider_id'),
+        Index('resource_providers_parent_provider_id_idx',
+              'parent_provider_id'),
         schema.UniqueConstraint('name',
             name='uniq_resource_providers0name')
     )
@@ -287,9 +318,16 @@ class ResourceProvider(API_BASE):
     uuid = Column(String(36), nullable=False)
     name = Column(Unicode(200), nullable=True)
     generation = Column(Integer, default=0)
-    can_host = Column(Integer, default=0)
+    # Represents the root of the "tree" that the provider belongs to
+    root_provider_id = Column(Integer, ForeignKey('resource_providers.id'),
+        nullable=True)
+    # The immediate parent provider of this provider, or NULL if there is no
+    # parent. If parent_provider_id == NULL then root_provider_id == id
+    parent_provider_id = Column(Integer, ForeignKey('resource_providers.id'),
+        nullable=True)
 
 
+# TODO(stephenfin): Remove this as it's now unused post-placement split
 class Inventory(API_BASE):
     """Represents a quantity of available resource."""
 
@@ -321,6 +359,7 @@ class Inventory(API_BASE):
         foreign_keys=resource_provider_id)
 
 
+# TODO(stephenfin): Remove this as it's now unused post-placement split
 class Allocation(API_BASE):
     """A use of inventory."""
 
@@ -346,6 +385,7 @@ class Allocation(API_BASE):
         foreign_keys=resource_provider_id)
 
 
+# TODO(stephenfin): Remove this as it's now unused post-placement split
 class ResourceProviderAggregate(API_BASE):
     """Associate a resource provider with an aggregate."""
 
@@ -359,6 +399,18 @@ class ResourceProviderAggregate(API_BASE):
     aggregate_id = Column(Integer, primary_key=True, nullable=False)
 
 
+# TODO(stephenfin): Remove this as it's now unused post-placement split
+class PlacementAggregate(API_BASE):
+    """A grouping of resource providers."""
+    __tablename__ = 'placement_aggregates'
+    __table_args__ = (
+        schema.UniqueConstraint("uuid", name="uniq_placement_aggregates0uuid"),
+    )
+
+    id = Column(Integer, primary_key=True, autoincrement=True)
+    uuid = Column(String(36), index=True)
+
+
 class InstanceGroupMember(API_BASE):
     """Represents the members for an instance group."""
     __tablename__ = 'instance_group_member'
@@ -381,6 +433,7 @@ class InstanceGroupPolicy(API_BASE):
     policy = Column(String(255))
     group_id = Column(Integer, ForeignKey('instance_groups.id'),
                       nullable=False)
+    rules = Column(Text)
 
 
 class InstanceGroup(API_BASE):
@@ -406,9 +459,216 @@ class InstanceGroup(API_BASE):
             primaryjoin='InstanceGroup.id == InstanceGroupMember.group_id')
 
     @property
-    def policies(self):
-        return [p.policy for p in self._policies]
+    def policy(self):
+        if len(self._policies) > 1:
+            msg = ("More than one policy (%(policies)s) is associated with "
+                   "group %(group_name)s, only the first one in the list "
+                   "would be returned.")
+            LOG.warning(msg, {"policies": [p.policy for p in self._policies],
+                              "group_name": self.name})
+        return self._policies[0] if self._policies else None
 
     @property
     def members(self):
         return [m.instance_uuid for m in self._members]
+
+
+class Quota(API_BASE):
+    """Represents a single quota override for a project.
+
+    If there is no row for a given project id and resource, then the
+    default for the quota class is used.  If there is no row for a
+    given quota class and resource, then the default for the
+    deployment is used. If the row is present but the hard limit is
+    Null, then the resource is unlimited.
+    """
+
+    __tablename__ = 'quotas'
+    __table_args__ = (
+        schema.UniqueConstraint("project_id", "resource",
+        name="uniq_quotas0project_id0resource"
+        ),
+    )
+    id = Column(Integer, primary_key=True)
+
+    project_id = Column(String(255))
+
+    resource = Column(String(255), nullable=False)
+    hard_limit = Column(Integer)
+
+
+class ProjectUserQuota(API_BASE):
+    """Represents a single quota override for a user with in a project."""
+
+    __tablename__ = 'project_user_quotas'
+    uniq_name = "uniq_project_user_quotas0user_id0project_id0resource"
+    __table_args__ = (
+        schema.UniqueConstraint("user_id", "project_id", "resource",
+                                name=uniq_name),
+        Index('project_user_quotas_project_id_idx',
+              'project_id'),
+        Index('project_user_quotas_user_id_idx',
+              'user_id',)
+    )
+    id = Column(Integer, primary_key=True, nullable=False)
+
+    project_id = Column(String(255), nullable=False)
+    user_id = Column(String(255), nullable=False)
+
+    resource = Column(String(255), nullable=False)
+    hard_limit = Column(Integer)
+
+
+class QuotaClass(API_BASE):
+    """Represents a single quota override for a quota class.
+
+    If there is no row for a given quota class and resource, then the
+    default for the deployment is used.  If the row is present but the
+    hard limit is Null, then the resource is unlimited.
+    """
+
+    __tablename__ = 'quota_classes'
+    __table_args__ = (
+        Index('quota_classes_class_name_idx', 'class_name'),
+    )
+    id = Column(Integer, primary_key=True)
+
+    class_name = Column(String(255))
+
+    resource = Column(String(255))
+    hard_limit = Column(Integer)
+
+
+class QuotaUsage(API_BASE):
+    """Represents the current usage for a given resource."""
+
+    __tablename__ = 'quota_usages'
+    __table_args__ = (
+        Index('quota_usages_project_id_idx', 'project_id'),
+        Index('quota_usages_user_id_idx', 'user_id'),
+    )
+    id = Column(Integer, primary_key=True)
+
+    project_id = Column(String(255))
+    user_id = Column(String(255))
+    resource = Column(String(255), nullable=False)
+
+    in_use = Column(Integer, nullable=False)
+    reserved = Column(Integer, nullable=False)
+
+    @property
+    def total(self):
+        return self.in_use + self.reserved
+
+    until_refresh = Column(Integer)
+
+
+class Reservation(API_BASE):
+    """Represents a resource reservation for quotas."""
+
+    __tablename__ = 'reservations'
+    __table_args__ = (
+        Index('reservations_project_id_idx', 'project_id'),
+        Index('reservations_uuid_idx', 'uuid'),
+        Index('reservations_expire_idx', 'expire'),
+        Index('reservations_user_id_idx', 'user_id'),
+    )
+    id = Column(Integer, primary_key=True, nullable=False)
+    uuid = Column(String(36), nullable=False)
+
+    usage_id = Column(Integer, ForeignKey('quota_usages.id'), nullable=False)
+
+    project_id = Column(String(255))
+    user_id = Column(String(255))
+    resource = Column(String(255))
+
+    delta = Column(Integer, nullable=False)
+    expire = Column(DateTime)
+
+    usage = orm.relationship(
+        "QuotaUsage",
+        foreign_keys=usage_id,
+        primaryjoin='Reservation.usage_id == QuotaUsage.id')
+
+
+class Trait(API_BASE):
+    """Represents a trait."""
+
+    __tablename__ = "traits"
+    __table_args__ = (
+        schema.UniqueConstraint('name', name='uniq_traits0name'),
+    )
+
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    name = Column(Unicode(255), nullable=False)
+
+
+# TODO(stephenfin): Remove this as it's now unused post-placement split
+class ResourceProviderTrait(API_BASE):
+    """Represents the relationship between traits and resource provider"""
+
+    __tablename__ = "resource_provider_traits"
+    __table_args__ = (
+        Index('resource_provider_traits_resource_provider_trait_idx',
+              'resource_provider_id', 'trait_id'),
+    )
+
+    trait_id = Column(Integer, ForeignKey('traits.id'), primary_key=True,
+                      nullable=False)
+    resource_provider_id = Column(Integer,
+                                  ForeignKey('resource_providers.id'),
+                                  primary_key=True,
+                                  nullable=False)
+
+
+# TODO(stephenfin): Remove this as it's unused
+class Project(API_BASE):
+    """The project is the Keystone project."""
+
+    __tablename__ = 'projects'
+    __table_args__ = (
+        schema.UniqueConstraint(
+            'external_id',
+            name='uniq_projects0external_id',
+        ),
+    )
+
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    external_id = Column(String(255), nullable=False)
+
+
+# TODO(stephenfin): Remove this as it's unused
+class User(API_BASE):
+    """The user is the Keystone user."""
+
+    __tablename__ = 'users'
+    __table_args__ = (
+        schema.UniqueConstraint(
+            'external_id',
+            name='uniq_users0external_id',
+        ),
+    )
+
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    external_id = Column(String(255), nullable=False)
+
+
+# TODO(stephenfin): Remove this as it's unused
+class Consumer(API_BASE):
+    """Represents a resource consumer."""
+
+    __tablename__ = 'consumers'
+    __table_args__ = (
+        Index('consumers_project_id_uuid_idx', 'project_id', 'uuid'),
+        Index('consumers_project_id_user_id_uuid_idx', 'project_id', 'user_id',
+              'uuid'),
+        schema.UniqueConstraint('uuid', name='uniq_consumers0uuid'),
+    )
+
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    uuid = Column(String(36), nullable=False)
+    project_id = Column(Integer, nullable=False)
+    user_id = Column(Integer, nullable=False)
+    # FIXME(mriedem): Change this to server_default=text("0") to match the
+    # 059_add_consumer_generation script once bug 1776527 is fixed.
+    generation = Column(Integer, nullable=False, server_default="0", default=0)
