@@ -21,8 +21,6 @@ from sqlalchemy import ForeignKey, Index, Integer, MetaData, String, Table
 from sqlalchemy import Text
 from sqlalchemy.types import NullType
 
-from nova.i18n import _LE
-
 LOG = logging.getLogger(__name__)
 
 
@@ -81,7 +79,7 @@ def _create_shadow_tables(migrate_engine):
             shadow_table.create()
         except Exception:
             LOG.info(repr(shadow_table))
-            LOG.exception(_LE('Exception while creating table.'))
+            LOG.exception('Exception while creating table.')
             raise
 
 
@@ -1075,7 +1073,7 @@ def upgrade(migrate_engine):
             table.create()
         except Exception:
             LOG.info(repr(table))
-            LOG.exception(_LE('Exception while creating table.'))
+            LOG.exception('Exception while creating table.')
             raise
 
     # task log unique constraint
@@ -1096,7 +1094,7 @@ def upgrade(migrate_engine):
     UniqueConstraint('flavorid', 'deleted', table=instance_types,
                      name='uniq_instance_types0flavorid0deleted').create()
 
-    # keypair contraint
+    # keypair constraint
     UniqueConstraint('user_id', 'name', 'deleted', table=key_pairs,
                      name='uniq_key_pairs0user_id0name0deleted').create()
 
@@ -1509,7 +1507,7 @@ def upgrade(migrate_engine):
             ])
 
     for fkey_pair in fkeys:
-        if migrate_engine.name in ('mysql'):
+        if migrate_engine.name == 'mysql':
             # For MySQL we name our fkeys explicitly
             # so they match Havana
             fkey = ForeignKeyConstraint(columns=fkey_pair[0],
@@ -1528,7 +1526,7 @@ def upgrade(migrate_engine):
             'ALTER TABLE migrate_version CONVERT TO CHARACTER SET utf8')
         # Set default DB charset to UTF8.
         migrate_engine.execute(
-            'ALTER DATABASE %s DEFAULT CHARACTER SET utf8' %
+            'ALTER DATABASE `%s` DEFAULT CHARACTER SET utf8' %
             migrate_engine.url.database)
 
     _create_shadow_tables(migrate_engine)
