@@ -12,22 +12,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import os
 import re
 
+from cursive import signature_utils
+from oslo_serialization import jsonutils
 from oslo_versionedobjects import fields
 import six
 
-# TODO(berrange) Temporary import for Arch class
-from nova.compute import arch
-# TODO(berrange) Temporary import for CPU* classes
-from nova.compute import cpumodel
-# TODO(berrange) Temporary import for HVType class
-from nova.compute import hv_type
-# TODO(berrange) Temporary import for VMMode class
-from nova.compute import vm_mode
 from nova import exception
 from nova.i18n import _
 from nova.network import model as network_model
+from nova.virt import arch
 
 
 # Import field errors from oslo.versionedobjects
@@ -39,8 +35,10 @@ ElementTypeError = fields.ElementTypeError
 BooleanField = fields.BooleanField
 UnspecifiedDefault = fields.UnspecifiedDefault
 IntegerField = fields.IntegerField
+NonNegativeIntegerField = fields.NonNegativeIntegerField
 UUIDField = fields.UUIDField
 FloatField = fields.FloatField
+NonNegativeFloatField = fields.NonNegativeFloatField
 StringField = fields.StringField
 SensitiveStringField = fields.SensitiveStringField
 EnumField = fields.EnumField
@@ -49,6 +47,7 @@ DictOfStringsField = fields.DictOfStringsField
 DictOfNullableStringsField = fields.DictOfNullableStringsField
 DictOfIntegersField = fields.DictOfIntegersField
 ListOfStringsField = fields.ListOfStringsField
+ListOfUUIDField = fields.ListOfUUIDField
 SetOfIntegersField = fields.SetOfIntegersField
 ListOfSetsOfIntegersField = fields.ListOfSetsOfIntegersField
 ListOfDictOfNullableStringsField = fields.ListOfDictOfNullableStringsField
@@ -61,18 +60,22 @@ DictOfListOfStringsField = fields.DictOfListOfStringsField
 IPAddressField = fields.IPAddressField
 IPV4AddressField = fields.IPV4AddressField
 IPV6AddressField = fields.IPV6AddressField
+IPV4AndV6AddressField = fields.IPV4AndV6AddressField
 IPNetworkField = fields.IPNetworkField
 IPV4NetworkField = fields.IPV4NetworkField
 IPV6NetworkField = fields.IPV6NetworkField
 AutoTypedField = fields.AutoTypedField
 BaseEnumField = fields.BaseEnumField
 MACAddressField = fields.MACAddressField
+ListOfIntegersField = fields.ListOfIntegersField
+PCIAddressField = fields.PCIAddressField
 
 
 # NOTE(danms): These are things we need to import for some of our
 # own implementations below, our tests, or other transitional
 # bits of code. These should be removable after we finish our
-# conversion
+# conversion. So do not use these nova fields directly in any new code;
+# instead, use the oslo.versionedobjects fields.
 Enum = fields.Enum
 Field = fields.Field
 FieldType = fields.FieldType
@@ -88,19 +91,138 @@ IPV4Network = fields.IPV4Network
 IPV6Network = fields.IPV6Network
 
 
+class ResourceClass(fields.StringPattern):
+
+    PATTERN = r"^[A-Z0-9_]+$"
+    _REGEX = re.compile(PATTERN)
+
+    @staticmethod
+    def coerce(obj, attr, value):
+        if isinstance(value, six.string_types):
+            uppered = value.upper()
+            if ResourceClass._REGEX.match(uppered):
+                return uppered
+        raise ValueError(_("Malformed Resource Class %s") % value)
+
+
+class ResourceClassField(AutoTypedField):
+    AUTO_TYPE = ResourceClass()
+
+
+class SetOfStringsField(AutoTypedField):
+    AUTO_TYPE = Set(fields.String())
+
+
 class BaseNovaEnum(Enum):
     def __init__(self, **kwargs):
         super(BaseNovaEnum, self).__init__(valid_values=self.__class__.ALL)
 
 
 class Architecture(BaseNovaEnum):
-    # TODO(berrange): move all constants out of 'nova.compute.arch'
-    # into fields on this class
+    """Represents CPU architectures.
+
+    Provides the standard names for all known processor architectures.
+    Many have multiple variants to deal with big-endian vs little-endian
+    modes, as well as 32 vs 64 bit word sizes. These names are chosen to
+    be identical to the architecture names expected by libvirt, so if
+    ever adding new ones then ensure it matches libvirt's expectation.
+    """
+
+    ALPHA = arch.ALPHA
+    ARMV6 = arch.ARMV6
+    ARMV7 = arch.ARMV7
+    ARMV7B = arch.ARMV7B
+
+    AARCH64 = arch.AARCH64
+    CRIS = arch.CRIS
+    I686 = arch.I686
+    IA64 = arch.IA64
+    LM32 = arch.LM32
+
+    M68K = arch.M68K
+    MICROBLAZE = arch.MICROBLAZE
+    MICROBLAZEEL = arch.MICROBLAZEEL
+    MIPS = arch.MIPS
+    MIPSEL = arch.MIPSEL
+
+    MIPS64 = arch.MIPS64
+    MIPS64EL = arch.MIPS64EL
+    OPENRISC = arch.OPENRISC
+    PARISC = arch.PARISC
+    PARISC64 = arch.PARISC64
+
+    PPC = arch.PPC
+    PPCLE = arch.PPCLE
+    PPC64 = arch.PPC64
+    PPC64LE = arch.PPC64LE
+    PPCEMB = arch.PPCEMB
+
+    S390 = arch.S390
+    S390X = arch.S390X
+    SH4 = arch.SH4
+    SH4EB = arch.SH4EB
+    SPARC = arch.SPARC
+
+    SPARC64 = arch.SPARC64
+    UNICORE32 = arch.UNICORE32
+    X86_64 = arch.X86_64
+    XTENSA = arch.XTENSA
+    XTENSAEB = arch.XTENSAEB
+
     ALL = arch.ALL
 
+    @classmethod
+    def from_host(cls):
+        """Get the architecture of the host OS
+
+        :returns: the canonicalized host architecture
+        """
+
+        return cls.canonicalize(os.uname()[4])
+
+    @classmethod
+    def is_valid(cls, name):
+        """Check if a string is a valid architecture
+
+        :param name: architecture name to validate
+
+        :returns: True if @name is valid
+        """
+
+        return name in cls.ALL
+
+    @classmethod
+    def canonicalize(cls, name):
+        """Canonicalize the architecture name
+
+        :param name: architecture name to canonicalize
+
+        :returns: a canonical architecture name
+        """
+
+        if name is None:
+            return None
+
+        newname = name.lower()
+
+        if newname in ("i386", "i486", "i586"):
+            newname = cls.I686
+
+        # Xen mistake from Icehouse or earlier
+        if newname in ("x86_32", "x86_32p"):
+            newname = cls.I686
+
+        if newname == "amd64":
+            newname = cls.X86_64
+
+        if not cls.is_valid(newname):
+            raise exception.InvalidArchitectureName(arch=name)
+
+        return newname
+
     def coerce(self, obj, attr, value):
         try:
-            value = arch.canonicalize(value)
+            value = self.canonicalize(value)
         except exception.InvalidArchitectureName:
             msg = _("Architecture name '%s' is not valid") % value
             raise ValueError(msg)
@@ -150,8 +272,9 @@ class CPUAllocationPolicy(BaseNovaEnum):
 
     DEDICATED = "dedicated"
     SHARED = "shared"
+    MIXED = "mixed"
 
-    ALL = (DEDICATED, SHARED)
+    ALL = (DEDICATED, SHARED, MIXED)
 
 
 class CPUThreadAllocationPolicy(BaseNovaEnum):
@@ -174,26 +297,52 @@ class CPUThreadAllocationPolicy(BaseNovaEnum):
     ALL = (PREFER, ISOLATE, REQUIRE)
 
 
+class CPUEmulatorThreadsPolicy(BaseNovaEnum):
+
+    # share (default): Emulator threads float across the pCPUs
+    # associated to the guest.
+    SHARE = "share"
+    # isolate: Emulator threads are isolated on a single pCPU.
+    ISOLATE = "isolate"
+
+    ALL = (SHARE, ISOLATE)
+
+
 class CPUMode(BaseNovaEnum):
-    # TODO(berrange): move all constants out of 'nova.compute.cpumodel'
-    # into fields on this class
-    ALL = cpumodel.ALL_CPUMODES
+
+    CUSTOM = 'custom'
+    HOST_MODEL = 'host-model'
+    HOST_PASSTHROUGH = 'host-passthrough'
+
+    ALL = (CUSTOM, HOST_MODEL, HOST_PASSTHROUGH)
 
 
 class CPUMatch(BaseNovaEnum):
-    # TODO(berrange): move all constants out of 'nova.compute.cpumodel'
-    # into fields on this class
-    ALL = cpumodel.ALL_MATCHES
+
+    MINIMUM = 'minimum'
+    EXACT = 'exact'
+    STRICT = 'strict'
+
+    ALL = (MINIMUM, EXACT, STRICT)
 
 
 class CPUFeaturePolicy(BaseNovaEnum):
-    # TODO(berrange): move all constants out of 'nova.compute.cpumodel'
-    # into fields on this class
-    ALL = cpumodel.ALL_POLICIES
+
+    FORCE = 'force'
+    REQUIRE = 'require'
+    OPTIONAL = 'optional'
+    DISABLE = 'disable'
+    FORBID = 'forbid'
+
+    ALL = (FORCE, REQUIRE, OPTIONAL, DISABLE, FORBID)
 
 
 class DiskBus(BaseNovaEnum):
 
+    # NOTE(aspiers): If you change this, don't forget to update the
+    # docs and metadata for hw_*_bus in glance.
+    # NOTE(lyarwood): Also update the possible values in the api-ref for the
+    # block_device_mapping_v2.disk_bus parameter.
     FDC = "fdc"
     IDE = "ide"
     SATA = "sata"
@@ -207,6 +356,18 @@ class DiskBus(BaseNovaEnum):
     ALL = (FDC, IDE, SATA, SCSI, USB, VIRTIO, XEN, LXC, UML)
 
 
+class DiskConfig(BaseNovaEnum):
+
+    MANUAL = "MANUAL"
+    AUTO = "AUTO"
+
+    ALL = (MANUAL, AUTO)
+
+    def coerce(self, obj, attr, value):
+        enum_value = DiskConfig.AUTO if value else DiskConfig.MANUAL
+        return super(DiskConfig, self).coerce(obj, attr, enum_value)
+
+
 class FirmwareType(BaseNovaEnum):
 
     UEFI = "uefi"
@@ -216,29 +377,103 @@ class FirmwareType(BaseNovaEnum):
 
 
 class HVType(BaseNovaEnum):
-    # TODO(berrange): move all constants out of 'nova.compute.hv_type'
-    # into fields on this class
-    ALL = hv_type.ALL
+    """Represents virtualization types.
+
+    Provide the standard names for all known guest virtualization
+    types. This is not to be confused with the Nova hypervisor driver
+    types, since one driver may support multiple virtualization types
+    and one virtualization type may be supported by multiple drivers.
+    """
+
+    BAREMETAL = 'baremetal'
+    BHYVE = 'bhyve'
+    DOCKER = 'docker'
+    FAKE = 'fake'
+    HYPERV = 'hyperv'
+    IRONIC = 'ironic'
+    KQEMU = 'kqemu'
+    KVM = 'kvm'
+    LXC = 'lxc'
+    LXD = 'lxd'
+    OPENVZ = 'openvz'
+    PARALLELS = 'parallels'
+    VIRTUOZZO = 'vz'
+    PHYP = 'phyp'
+    QEMU = 'qemu'
+    TEST = 'test'
+    UML = 'uml'
+    VBOX = 'vbox'
+    VMWARE = 'vmware'
+    XEN = 'xen'
+    ZVM = 'zvm'
+    PRSM = 'prsm'
+
+    ALL = (BAREMETAL, BHYVE, DOCKER, FAKE, HYPERV, IRONIC, KQEMU, KVM, LXC,
+           LXD, OPENVZ, PARALLELS, PHYP, QEMU, TEST, UML, VBOX, VIRTUOZZO,
+           VMWARE, XEN, ZVM, PRSM)
 
     def coerce(self, obj, attr, value):
         try:
-            value = hv_type.canonicalize(value)
+            value = self.canonicalize(value)
         except exception.InvalidHypervisorVirtType:
             msg = _("Hypervisor virt type '%s' is not valid") % value
             raise ValueError(msg)
+
         return super(HVType, self).coerce(obj, attr, value)
 
+    @classmethod
+    def is_valid(cls, name):
+        """Check if a string is a valid hypervisor type
+
+        :param name: hypervisor type name to validate
+
+        :returns: True if @name is valid
+        """
+        return name in cls.ALL
+
+    @classmethod
+    def canonicalize(cls, name):
+        """Canonicalize the hypervisor type name
+
+        :param name: hypervisor type name to canonicalize
+
+        :returns: a canonical hypervisor type name
+        """
+        if name is None:
+            return None
+
+        newname = name.lower()
+
+        if newname == 'xapi':
+            newname = cls.XEN
+
+        if not cls.is_valid(newname):
+            raise exception.InvalidHypervisorVirtType(hv_type=name)
+
+        return newname
+
 
 class ImageSignatureHashType(BaseNovaEnum):
     # Represents the possible hash methods used for image signing
-    ALL = ('SHA-224', 'SHA-256', 'SHA-384', 'SHA-512')
+    ALL = tuple(sorted(signature_utils.HASH_METHODS.keys()))
 
 
 class ImageSignatureKeyType(BaseNovaEnum):
     # Represents the possible keypair types used for image signing
-    ALL = ('DSA', 'ECC_SECT571K1', 'ECC_SECT409K1', 'ECC_SECT571R1',
-           'ECC_SECT409R1', 'ECC_SECP521R1', 'ECC_SECP384R1', 'RSA-PSS'
-           )
+    ALL = (
+        'DSA', 'ECC_SECP384R1', 'ECC_SECP521R1', 'ECC_SECT409K1',
+        'ECC_SECT409R1', 'ECC_SECT571K1', 'ECC_SECT571R1', 'RSA-PSS'
+    )
+
+
+class MigrationType(BaseNovaEnum):
+
+    MIGRATION = 'migration'  # cold migration
+    RESIZE = 'resize'
+    LIVE_MIGRATION = 'live-migration'
+    EVACUATION = 'evacuation'
+
+    ALL = (MIGRATION, RESIZE, LIVE_MIGRATION, EVACUATION)
 
 
 class OSType(BaseNovaEnum):
@@ -255,43 +490,42 @@ class OSType(BaseNovaEnum):
         return super(OSType, self).coerce(obj, attr, value)
 
 
-class ResourceClass(BaseNovaEnum):
-    """Classes of resources provided to consumers."""
-
-    VCPU = 'VCPU'
-    MEMORY_MB = 'MEMORY_MB'
-    DISK_GB = 'DISK_GB'
-    PCI_DEVICE = 'PCI_DEVICE'
-    SRIOV_NET_VF = 'SRIOV_NET_VF'
-    NUMA_SOCKET = 'NUMA_SOCKET'
-    NUMA_CORE = 'NUMA_CORE'
-    NUMA_THREAD = 'NUMA_THREAD'
-    NUMA_MEMORY_MB = 'NUMA_MEMORY_MB'
-    IPV4_ADDRESS = 'IPV4_ADDRESS'
-
-    # The ordering here is relevant. If you must add a value, only
-    # append.
-    ALL = (VCPU, MEMORY_MB, DISK_GB, PCI_DEVICE, SRIOV_NET_VF, NUMA_SOCKET,
-           NUMA_CORE, NUMA_THREAD, NUMA_MEMORY_MB, IPV4_ADDRESS)
-
-    @classmethod
-    def index(cls, value):
-        """Return an index into the Enum given a value."""
-        return cls.ALL.index(value)
-
-    @classmethod
-    def from_index(cls, index):
-        """Return the Enum value at a given index."""
-        return cls.ALL[index]
-
-
 class RNGModel(BaseNovaEnum):
 
+    # NOTE(kchamart): Along with "virtio", we may need to extend this (if a
+    # good reason shows up) to allow two more values for VirtIO
+    # transitional and non-transitional devices (available since libvirt
+    # 5.2.0):
+    #
+    #   - virtio-transitional
+    #   - virtio-nontransitional
+    #
+    # This allows one to choose whether you want to have compatibility
+    # with older guest operating systems.  The value you select will in
+    # turn decide the kind of PCI topology the guest will get.
+    #
+    # Details:
+    # https://libvirt.org/formatdomain.html#elementsVirtioTransitional
     VIRTIO = "virtio"
 
     ALL = (VIRTIO,)
 
 
+class TPMModel(BaseNovaEnum):
+
+    TIS = "tpm-tis"
+    CRB = "tpm-crb"
+
+    ALL = (TIS, CRB)
+
+
+class TPMVersion(BaseNovaEnum):
+    v1_2 = "1.2"
+    v2_0 = "2.0"
+
+    ALL = (v1_2, v2_0)
+
+
 class SCSIModel(BaseNovaEnum):
 
     BUSLOGIC = "buslogic"
@@ -333,8 +567,11 @@ class VideoModel(BaseNovaEnum):
     VGA = "vga"
     VMVGA = "vmvga"
     XEN = "xen"
+    VIRTIO = 'virtio'
+    GOP = 'gop'
+    NONE = 'none'
 
-    ALL = (CIRRUS, QXL, VGA, VMVGA, XEN)
+    ALL = (CIRRUS, QXL, VGA, VMVGA, XEN, VIRTIO, GOP, NONE)
 
 
 class VIFModel(BaseNovaEnum):
@@ -355,9 +592,6 @@ class VIFModel(BaseNovaEnum):
 
     ALL = network_model.VIF_MODEL_ALL
 
-    def _get_legacy(self, value):
-        return value
-
     def coerce(self, obj, attr, value):
         # Some compat for strings we'd see in the legacy
         # hw_vif_model image property
@@ -367,18 +601,80 @@ class VIFModel(BaseNovaEnum):
 
 
 class VMMode(BaseNovaEnum):
-    # TODO(berrange): move all constants out of 'nova.compute.vm_mode'
-    # into fields on this class
-    ALL = vm_mode.ALL
+    """Represents possible vm modes for instances.
+
+    Compute instance VM modes represent the host/guest ABI used for the
+    virtual machine or container. Individual hypervisors may support
+    multiple different vm modes per host. Available VM modes for a
+    hypervisor driver may also vary according to the architecture it is
+    running on.
+    """
+    HVM = 'hvm'  # Native ABI (aka fully virtualized)
+    XEN = 'xen'  # Xen 3.0 paravirtualized
+    UML = 'uml'  # User Mode Linux paravirtualized
+    EXE = 'exe'  # Executables in containers
+
+    ALL = (HVM, XEN, UML, EXE)
 
     def coerce(self, obj, attr, value):
         try:
-            value = vm_mode.canonicalize(value)
+            value = self.canonicalize(value)
         except exception.InvalidVirtualMachineMode:
             msg = _("Virtual machine mode '%s' is not valid") % value
             raise ValueError(msg)
+
         return super(VMMode, self).coerce(obj, attr, value)
 
+    @classmethod
+    def get_from_instance(cls, instance):
+        """Get the vm mode for an instance
+
+        :param instance: instance object to query
+
+        :returns: canonicalized vm mode for the instance
+        """
+        mode = instance.vm_mode
+
+        return cls.canonicalize(mode)
+
+    @classmethod
+    def is_valid(cls, name):
+        """Check if a string is a valid vm mode
+
+        :param name: vm mode name to validate
+
+        :returns: True if @name is valid
+        """
+        return name in cls.ALL
+
+    @classmethod
+    def canonicalize(cls, mode):
+        """Canonicalize the vm mode
+
+        :param name: vm mode name to canonicalize
+
+        :returns: a canonical vm mode name
+        """
+        if mode is None:
+            return None
+
+        mode = mode.lower()
+
+        # For compatibility with pre-Folsom deployments
+        if mode == 'pv':
+            mode = cls.XEN
+
+        if mode == 'hv':
+            mode = cls.HVM
+
+        if mode == 'baremetal':
+            mode = cls.HVM
+
+        if not cls.is_valid(mode):
+            raise exception.InvalidVirtualMachineMode(vmmode=mode)
+
+        return mode
+
 
 class WatchdogAction(BaseNovaEnum):
 
@@ -386,8 +682,9 @@ class WatchdogAction(BaseNovaEnum):
     PAUSE = "pause"
     POWEROFF = "poweroff"
     RESET = "reset"
+    DISABLED = "disabled"
 
-    ALL = (NONE, PAUSE, POWEROFF, RESET)
+    ALL = (NONE, PAUSE, POWEROFF, RESET, DISABLED)
 
 
 class MonitorMetricType(BaseNovaEnum):
@@ -457,6 +754,15 @@ class PciDeviceType(BaseNovaEnum):
     ALL = (STANDARD, SRIOV_PF, SRIOV_VF)
 
 
+class PCINUMAAffinityPolicy(BaseNovaEnum):
+
+    REQUIRED = "required"
+    LEGACY = "legacy"
+    PREFERRED = "preferred"
+
+    ALL = (REQUIRED, LEGACY, PREFERRED)
+
+
 class DiskFormat(BaseNovaEnum):
     RBD = "rbd"
     LVM = "lvm"
@@ -471,6 +777,18 @@ class DiskFormat(BaseNovaEnum):
     ALL = (RBD, LVM, QCOW2, RAW, PLOOP, VHD, VMDK, VDI, ISO)
 
 
+# TODO(stephenfin): Remove the xenapi value when we bump the 'Diagnostics'
+# object (the only user of this enum) to 2.0
+class HypervisorDriver(BaseNovaEnum):
+    LIBVIRT = "libvirt"
+    XENAPI = "xenapi"
+    VMWAREAPI = "vmwareapi"
+    IRONIC = "ironic"
+    HYPERV = "hyperv"
+
+    ALL = (LIBVIRT, XENAPI, VMWAREAPI, IRONIC, HYPERV)
+
+
 class PointerModelType(BaseNovaEnum):
 
     USBTABLET = "usbtablet"
@@ -494,8 +812,48 @@ class NotificationPhase(BaseNovaEnum):
     START = 'start'
     END = 'end'
     ERROR = 'error'
+    PROGRESS = 'progress'
 
-    ALL = (START, END, ERROR)
+    ALL = (START, END, ERROR, PROGRESS)
+
+
+class NotificationSource(BaseNovaEnum):
+    """Represents possible nova binary service names in notification envelope.
+
+    The publisher_id field of the nova notifications consists of the name of
+    the host and the name of the service binary that emits the notification.
+    The below values are the ones that is used in every notification. Please
+    note that on the REST API the nova-api service binary is called
+    nova-osapi_compute. This is not reflected here as notifications always used
+    the name nova-api instead.
+    """
+
+    COMPUTE = 'nova-compute'
+    API = 'nova-api'
+    CONDUCTOR = 'nova-conductor'
+    SCHEDULER = 'nova-scheduler'
+    # TODO(stephenfin): Remove 'NETWORK' when 'NotificationPublisher' is
+    # updated to version 3.0
+    NETWORK = 'nova-network'
+    # TODO(stephenfin): Remove 'CONSOLEAUTH' when 'NotificationPublisher' is
+    # updated to version 3.0
+    CONSOLEAUTH = 'nova-consoleauth'
+    # TODO(stephenfin): Remove when 'NotificationPublisher' object version is
+    # bumped to 3.0
+    CELLS = 'nova-cells'
+    # TODO(stephenfin): Remove when 'NotificationPublisher' object version is
+    # bumped to 3.0
+    CONSOLE = 'nova-console'
+    METADATA = 'nova-metadata'
+
+    ALL = (API, COMPUTE, CONDUCTOR, SCHEDULER,
+           NETWORK, CONSOLEAUTH, CELLS, CONSOLE, METADATA)
+
+    @staticmethod
+    def get_source_by_binary(binary):
+        # nova-osapi_compute binary name needs to be translated to nova-api
+        # notification source enum value.
+        return "nova-api" if binary == "nova-osapi_compute" else binary
 
 
 class NotificationAction(BaseNovaEnum):
@@ -512,14 +870,64 @@ class NotificationAction(BaseNovaEnum):
     REBOOT = 'reboot'
     SHUTDOWN = 'shutdown'
     SNAPSHOT = 'snapshot'
-    ADD_FIXED_IP = 'add_fixed_ip'
+    INTERFACE_ATTACH = 'interface_attach'
     SHELVE = 'shelve'
     RESUME = 'resume'
     RESTORE = 'restore'
+    EXISTS = 'exists'
+    RESCUE = 'rescue'
+    VOLUME_ATTACH = 'volume_attach'
+    VOLUME_DETACH = 'volume_detach'
+    CREATE = 'create'
+    IMPORT = 'import'
+    EVACUATE = 'evacuate'
+    RESIZE_FINISH = 'resize_finish'
+    LIVE_MIGRATION_ABORT = 'live_migration_abort'
+    LIVE_MIGRATION_POST_DEST = 'live_migration_post_dest'
+    LIVE_MIGRATION_POST = 'live_migration_post'
+    LIVE_MIGRATION_PRE = 'live_migration_pre'
+    LIVE_MIGRATION_ROLLBACK_DEST = 'live_migration_rollback_dest'
+    LIVE_MIGRATION_ROLLBACK = 'live_migration_rollback'
+    LIVE_MIGRATION_FORCE_COMPLETE = 'live_migration_force_complete'
+    REBUILD = 'rebuild'
+    REBUILD_SCHEDULED = 'rebuild_scheduled'
+    INTERFACE_DETACH = 'interface_detach'
+    RESIZE_CONFIRM = 'resize_confirm'
+    RESIZE_PREP = 'resize_prep'
+    RESIZE_REVERT = 'resize_revert'
+    SELECT_DESTINATIONS = 'select_destinations'
+    SHELVE_OFFLOAD = 'shelve_offload'
+    SOFT_DELETE = 'soft_delete'
+    TRIGGER_CRASH_DUMP = 'trigger_crash_dump'
+    UNRESCUE = 'unrescue'
+    UNSHELVE = 'unshelve'
+    ADD_HOST = 'add_host'
+    REMOVE_HOST = 'remove_host'
+    ADD_MEMBER = 'add_member'
+    UPDATE_METADATA = 'update_metadata'
+    LOCK = 'lock'
+    UNLOCK = 'unlock'
+    UPDATE_PROP = 'update_prop'
+    CONNECT = 'connect'
+    USAGE = 'usage'
+    BUILD_INSTANCES = 'build_instances'
+    MIGRATE_SERVER = 'migrate_server'
+    REBUILD_SERVER = 'rebuild_server'
+    IMAGE_CACHE = 'cache_images'
 
     ALL = (UPDATE, EXCEPTION, DELETE, PAUSE, UNPAUSE, RESIZE, VOLUME_SWAP,
-           SUSPEND, POWER_ON, REBOOT, SHUTDOWN, SNAPSHOT, ADD_FIXED_IP,
-           POWER_OFF, SHELVE, RESUME, RESTORE)
+           SUSPEND, POWER_ON, REBOOT, SHUTDOWN, SNAPSHOT, INTERFACE_ATTACH,
+           POWER_OFF, SHELVE, RESUME, RESTORE, EXISTS, RESCUE, VOLUME_ATTACH,
+           VOLUME_DETACH, CREATE, IMPORT, EVACUATE, RESIZE_FINISH,
+           LIVE_MIGRATION_ABORT, LIVE_MIGRATION_POST_DEST, LIVE_MIGRATION_POST,
+           LIVE_MIGRATION_PRE, LIVE_MIGRATION_ROLLBACK,
+           LIVE_MIGRATION_ROLLBACK_DEST, REBUILD, INTERFACE_DETACH,
+           RESIZE_CONFIRM, RESIZE_PREP, RESIZE_REVERT, SHELVE_OFFLOAD,
+           SOFT_DELETE, TRIGGER_CRASH_DUMP, UNRESCUE, UNSHELVE, ADD_HOST,
+           REMOVE_HOST, ADD_MEMBER, UPDATE_METADATA, LOCK, UNLOCK,
+           REBUILD_SCHEDULED, UPDATE_PROP, LIVE_MIGRATION_FORCE_COMPLETE,
+           CONNECT, USAGE, BUILD_INSTANCES, MIGRATE_SERVER, REBUILD_SERVER,
+           SELECT_DESTINATIONS, IMAGE_CACHE)
 
 
 # TODO(rlrossit): These should be changed over to be a StateMachine enum from
@@ -647,17 +1055,6 @@ class InstancePowerState(Enum):
         return cls.ALL[index]
 
 
-class IPV4AndV6Address(IPAddress):
-    @staticmethod
-    def coerce(obj, attr, value):
-        result = IPAddress.coerce(obj, attr, value)
-        if result.version != 4 and result.version != 6:
-            raise ValueError(_('Network "%(val)s" is not valid '
-                               'in field %(attr)s') %
-                             {'val': value, 'attr': attr})
-        return result
-
-
 class NetworkModel(FieldType):
     @staticmethod
     def coerce(obj, attr, value):
@@ -682,23 +1079,33 @@ class NetworkModel(FieldType):
         return 'NetworkModel(%s)' % (
             ','.join([str(vif['id']) for vif in value]))
 
+    def get_schema(self):
+        return {'type': ['string']}
+
+
+class NetworkVIFModel(FieldType):
+    """Represents a nova.network.model.VIF object, which is a dict of stuff."""
 
-class NonNegativeFloat(FieldType):
     @staticmethod
     def coerce(obj, attr, value):
-        v = float(value)
-        if v < 0:
-            raise ValueError(_('Value must be >= 0 for field %s') % attr)
-        return v
+        if isinstance(value, network_model.VIF):
+            return value
+        elif isinstance(value, six.string_types):
+            return NetworkVIFModel.from_primitive(obj, attr, value)
+        else:
+            raise ValueError(_('A nova.network.model.VIF object is required '
+                               'in field %s') % attr)
 
-
-class NonNegativeInteger(FieldType):
     @staticmethod
-    def coerce(obj, attr, value):
-        v = int(value)
-        if v < 0:
-            raise ValueError(_('Value must be >= 0 for field %s') % attr)
-        return v
+    def to_primitive(obj, attr, value):
+        return jsonutils.dumps(value)
+
+    @staticmethod
+    def from_primitive(obj, attr, value):
+        return network_model.VIF.hydrate(jsonutils.loads(value))
+
+    def get_schema(self):
+        return {'type': ['string']}
 
 
 class AddressBase(FieldType):
@@ -709,13 +1116,8 @@ class AddressBase(FieldType):
         else:
             raise ValueError(_('Value must match %s') % obj.PATTERN)
 
-
-class PCIAddress(AddressBase):
-    PATTERN = '[a-f0-9]{4}:[a-f0-9]{2}:[a-f0-9]{2}.[a-f0-9]'
-
-    @staticmethod
-    def coerce(obj, attr, value):
-        return AddressBase.coerce(PCIAddress, attr, value)
+    def get_schema(self):
+        return {'type': ['string'], 'pattern': self.PATTERN}
 
 
 class USBAddress(AddressBase):
@@ -742,8 +1144,12 @@ class IDEAddress(AddressBase):
         return AddressBase.coerce(IDEAddress, attr, value)
 
 
-class PCIAddressField(AutoTypedField):
-    AUTO_TYPE = PCIAddress()
+class XenAddress(AddressBase):
+    PATTERN = '(00[0-9]{2}00)|[1-9][0-9]+'
+
+    @staticmethod
+    def coerce(obj, attr, value):
+        return AddressBase.coerce(XenAddress, attr, value)
 
 
 class USBAddressField(AutoTypedField):
@@ -758,6 +1164,10 @@ class IDEAddressField(AutoTypedField):
     AUTO_TYPE = IDEAddress()
 
 
+class XenAddressField(AutoTypedField):
+    AUTO_TYPE = XenAddress()
+
+
 class ArchitectureField(BaseEnumField):
     AUTO_TYPE = Architecture()
 
@@ -786,6 +1196,10 @@ class CPUThreadAllocationPolicyField(BaseEnumField):
     AUTO_TYPE = CPUThreadAllocationPolicy()
 
 
+class CPUEmulatorThreadsPolicyField(BaseEnumField):
+    AUTO_TYPE = CPUEmulatorThreadsPolicy()
+
+
 class CPUModeField(BaseEnumField):
     AUTO_TYPE = CPUMode()
 
@@ -802,6 +1216,10 @@ class DiskBusField(BaseEnumField):
     AUTO_TYPE = DiskBus()
 
 
+class DiskConfigField(BaseEnumField):
+    AUTO_TYPE = DiskConfig()
+
+
 class FirmwareTypeField(BaseEnumField):
     AUTO_TYPE = FirmwareType()
 
@@ -818,26 +1236,26 @@ class ImageSignatureKeyTypeField(BaseEnumField):
     AUTO_TYPE = ImageSignatureKeyType()
 
 
+class MigrationTypeField(BaseEnumField):
+    AUTO_TYPE = MigrationType()
+
+
 class OSTypeField(BaseEnumField):
     AUTO_TYPE = OSType()
 
 
-class ResourceClassField(BaseEnumField):
-    AUTO_TYPE = ResourceClass()
-
-    def index(self, value):
-        """Return an index into the Enum given a value."""
-        return self._type.index(value)
-
-    def from_index(self, index):
-        """Return the Enum value at a given index."""
-        return self._type.from_index(index)
-
-
 class RNGModelField(BaseEnumField):
     AUTO_TYPE = RNGModel()
 
 
+class TPMModelField(BaseEnumField):
+    AUTO_TYPE = TPMModel()
+
+
+class TPMVersionField(BaseEnumField):
+    AUTO_TYPE = TPMVersion()
+
+
 class SCSIModelField(BaseEnumField):
     AUTO_TYPE = SCSIModel()
 
@@ -874,10 +1292,18 @@ class PciDeviceTypeField(BaseEnumField):
     AUTO_TYPE = PciDeviceType()
 
 
+class PCINUMAAffinityPolicyField(BaseEnumField):
+    AUTO_TYPE = PCINUMAAffinityPolicy()
+
+
 class DiskFormatField(BaseEnumField):
     AUTO_TYPE = DiskFormat()
 
 
+class HypervisorDriverField(BaseEnumField):
+    AUTO_TYPE = HypervisorDriver()
+
+
 class PointerModelField(BaseEnumField):
     AUTO_TYPE = PointerModelType()
 
@@ -894,6 +1320,10 @@ class NotificationActionField(BaseEnumField):
     AUTO_TYPE = NotificationAction()
 
 
+class NotificationSourceField(BaseEnumField):
+    AUTO_TYPE = NotificationSource()
+
+
 class InstanceStateField(BaseEnumField):
     AUTO_TYPE = InstanceState()
 
@@ -906,17 +1336,9 @@ class InstancePowerStateField(BaseEnumField):
     AUTO_TYPE = InstancePowerState()
 
 
-class IPV4AndV6AddressField(AutoTypedField):
-    AUTO_TYPE = IPV4AndV6Address()
+class ListOfListsOfStringsField(AutoTypedField):
+    AUTO_TYPE = List(List(fields.String()))
 
 
-class ListOfIntegersField(AutoTypedField):
-    AUTO_TYPE = List(fields.Integer())
-
-
-class NonNegativeFloatField(AutoTypedField):
-    AUTO_TYPE = NonNegativeFloat()
-
-
-class NonNegativeIntegerField(AutoTypedField):
-    AUTO_TYPE = NonNegativeInteger()
+class DictOfSetOfIntegersField(AutoTypedField):
+    AUTO_TYPE = Dict(Set(fields.Integer()))
