@@ -15,6 +15,7 @@
 import copy
 
 from oslo_utils import versionutils
+import six
 
 from nova import exception
 from nova import objects
@@ -168,12 +169,51 @@ class ImageMetaProps(base.NovaObject):
     # Version 1.17: Add lan9118 as valid nic for hw_vif_model property for qemu
     # Version 1.18: Pull signature properties from cursive library
     # Version 1.19: Added 'img_hide_hypervisor_id' type field
-    VERSION = '1.19'
+    # Version 1.20: Added 'traits_required' list field
+    # Version 1.21: Added 'hw_time_hpet' field
+    # Version 1.22: Added 'gop', 'virtio' and 'none' to hw_video_model field
+    # Version 1.23: Added 'hw_pmu' field
+    # Version 1.24: Added 'hw_mem_encryption' field
+    # Version 1.25: Added 'hw_pci_numa_affinity_policy' field
+    # Version 1.26: Added 'mixed' to 'hw_cpu_policy' field
+    # Version 1.27: Added 'hw_tpm_model' and 'hw_tpm_version' fields
+    # NOTE(efried): When bumping this version, the version of
+    # ImageMetaPropsPayload must also be bumped. See its docstring for details.
+    VERSION = '1.27'
 
     def obj_make_compatible(self, primitive, target_version):
         super(ImageMetaProps, self).obj_make_compatible(primitive,
                                                         target_version)
         target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version < (1, 27):
+            primitive.pop('hw_tpm_model', None)
+            primitive.pop('hw_tpm_version', None)
+        if target_version < (1, 26):
+            policy = primitive.get('hw_cpu_policy', None)
+            if policy == fields.CPUAllocationPolicy.MIXED:
+                raise exception.ObjectActionError(
+                    action='obj_make_compatible',
+                    reason='hw_cpu_policy=%s not supported in version %s' %
+                           (policy, target_version))
+        if target_version < (1, 25):
+            primitive.pop('hw_pci_numa_affinity_policy', None)
+        if target_version < (1, 24):
+            primitive.pop('hw_mem_encryption', None)
+        if target_version < (1, 23):
+            primitive.pop('hw_pmu', None)
+        # NOTE(sean-k-mooney): unlike other nova object we version this object
+        # when composed object are updated.
+        if target_version < (1, 22):
+            video = primitive.get('hw_video_model', None)
+            if video in ('gop', 'virtio', 'none'):
+                raise exception.ObjectActionError(
+                    action='obj_make_compatible',
+                    reason='hw_video_model=%s not supported in version %s' % (
+                        video, target_version))
+        if target_version < (1, 21):
+            primitive.pop('hw_time_hpet', None)
+        if target_version < (1, 20):
+            primitive.pop('traits_required', None)
         if target_version < (1, 19):
             primitive.pop('img_hide_hypervisor_id', None)
         if target_version < (1, 16) and 'hw_watchdog_action' in primitive:
@@ -221,6 +261,7 @@ class ImageMetaProps(base.NovaObject):
     # 'hw_' - settings affecting the guest virtual machine hardware
     # 'img_' - settings affecting the use of images by the compute node
     # 'os_' - settings affecting the guest operating system setup
+    # 'traits_required' - The required traits associated with the image
 
     fields = {
         # name of guest hardware architecture eg i686, x86_64, ppc64
@@ -291,6 +332,10 @@ class ImageMetaProps(base.NovaObject):
         # form string
         'hw_machine_type': fields.StringField(),
 
+        # boolean indicating that the guest needs to be booted with
+        # encrypted memory
+        'hw_mem_encryption': fields.FlexibleBooleanField(),
+
         # One of the magic strings 'small', 'any', 'large'
         # or an explicit page size in KB (eg 4, 2048, ...)
         'hw_mem_page_size': fields.StringField(),
@@ -306,9 +351,16 @@ class ImageMetaProps(base.NovaObject):
         # list value indicates the memory size of that node.
         'hw_numa_mem': fields.ListOfIntegersField(),
 
+        # Enum field to specify pci device NUMA affinity.
+        'hw_pci_numa_affinity_policy': fields.PCINUMAAffinityPolicyField(),
+
         # Generic property to specify the pointer model type.
         'hw_pointer_model': fields.PointerModelField(),
 
+        # boolean 'true' or 'false' to enable virtual performance
+        # monitoring unit (vPMU).
+        'hw_pmu': fields.FlexibleBooleanField(),
+
         # boolean 'yes' or 'no' to enable QEMU guest agent
         'hw_qemu_guest_agent': fields.FlexibleBooleanField(),
 
@@ -319,8 +371,17 @@ class ImageMetaProps(base.NovaObject):
         'hw_rescue_device': fields.BlockDeviceTypeField(),
 
         # name of the RNG device type eg virtio
+        # NOTE(kchamart): Although this is currently not used anymore,
+        # we should not remove / deprecate it yet, as we are likely to
+        # extend this field to allow two more values to support "VirtIO
+        # transitional/non-transitional devices" (refer to the note in
+        # RNGModel() class in nova/objects/fields.py), and thus expose
+        # to the user again.
         'hw_rng_model': fields.RNGModelField(),
 
+        # boolean 'true' or 'false' to enable HPET
+        'hw_time_hpet': fields.FlexibleBooleanField(),
+
         # number of serial ports to create
         'hw_serial_port_count': fields.IntegerField(),
 
@@ -346,6 +407,11 @@ class ImageMetaProps(base.NovaObject):
         # boolean - If true, this will enable the virtio-multiqueue feature
         'hw_vif_multiqueue_enabled': fields.FlexibleBooleanField(),
 
+        # name of emulated TPM model to use.
+        'hw_tpm_model': fields.TPMModelField(),
+        # version of emulated TPM to use.
+        'hw_tpm_version': fields.TPMVersionField(),
+
         # if true download using bittorrent
         'img_bittorrent': fields.FlexibleBooleanField(),
 
@@ -457,6 +523,13 @@ class ImageMetaProps(base.NovaObject):
         # is a fairly generic type. For a detailed type consider os_distro
         # instead
         'os_type': fields.OSTypeField(),
+
+        # The required traits associated with the image. Traits are expected to
+        # be defined as starting with `trait:` like below:
+        # trait:HW_CPU_X86_AVX2=required
+        # for trait in image_meta.traits_required:
+        # will yield trait strings such as 'HW_CPU_X86_AVX2'
+        'traits_required': fields.ListOfStringsField(),
     }
 
     # The keys are the legacy property names and
@@ -543,16 +616,26 @@ class ImageMetaProps(base.NovaObject):
             elif key == "hw_numa_cpus":
                 self._set_numa_cpus(image_props)
             else:
-                if key not in image_props:
+                # traits_required will be populated by
+                # _set_attr_from_trait_names
+                if key not in image_props or key == "traits_required":
                     continue
 
                 setattr(self, key, image_props[key])
 
+    def _set_attr_from_trait_names(self, image_props):
+        for trait in [six.text_type(k[6:]) for k, v in image_props.items()
+                      if six.text_type(k).startswith("trait:") and
+                      six.text_type(v) == six.text_type('required')]:
+            if 'traits_required' not in self:
+                self.traits_required = []
+            self.traits_required.append(trait)
+
     @classmethod
     def from_dict(cls, image_props):
         """Create instance from image properties dict
 
-        :param image_props: dictionary of image metdata properties
+        :param image_props: dictionary of image metadata properties
 
         Creates a new object instance, initializing from a
         dictionary of image metadata properties
@@ -567,6 +650,8 @@ class ImageMetaProps(base.NovaObject):
         # associated with the current name takes priority
         obj._set_attr_from_legacy_names(image_props)
         obj._set_attr_from_current_names(image_props)
+        obj._set_attr_from_trait_names(image_props)
+
         return obj
 
     def get(self, name, defvalue=None):
