@@ -385,7 +385,15 @@ class InstanceHelperMixin:
         """
         # if forcing the server onto a host, we have to use the admin API
         if not api:
-            api = self.api if not az else getattr(self, 'admin_api', self.api)
+            api = self.api if not az and not host else getattr(
+                self, 'admin_api', self.api)
+
+        if host and not api.microversion:
+            api.microversion = '2.74'
+            # with 2.74 networks param needs to use 'none' instead of None
+            # if no network is needed
+            if networks is None:
+                networks = 'none'
 
         body = self._build_server(
             name, image_uuid, flavor_id, networks, az, host)
