@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import mock
+from oslo_utils.fixture import uuidsentinel
 import testtools
 import webob.exc
 
@@ -20,12 +22,12 @@ from nova.api.openstack.compute import hosts as os_hosts_v21
 from nova.compute import power_state
 from nova.compute import vm_states
 from nova import context as context_maker
-from nova import db
+from nova.db import api as db
 from nova import exception
 from nova import test
+from nova.tests import fixtures
 from nova.tests.unit.api.openstack import fakes
 from nova.tests.unit import fake_hosts
-from nova.tests import uuidsentinel
 
 
 def stub_service_get_all(context, disabled=None):
@@ -38,7 +40,7 @@ def stub_service_get_by_host_and_binary(context, host_name, binary):
             return service
 
 
-def stub_set_host_enabled(context, host_name, enabled):
+def stub_set_host_enabled(self, context, host_name, enabled):
     """Simulates three possible behaviours for VM drivers or compute
     drivers when enabling or disabling a host.
 
@@ -63,7 +65,7 @@ def stub_set_host_enabled(context, host_name, enabled):
         return results[enabled]
 
 
-def stub_set_host_maintenance(context, host_name, mode):
+def stub_set_host_maintenance(self, context, host_name, mode):
     # We'll simulate success and failure by assuming
     # that 'host_c1' always succeeds, and 'host_c2'
     # always fails
@@ -85,7 +87,7 @@ def stub_set_host_maintenance(context, host_name, mode):
         return results[mode]
 
 
-def stub_host_power_action(context, host_name, action):
+def stub_host_power_action(self, context, host_name, action):
     if host_name == "notimplemented":
         raise NotImplementedError()
     elif host_name == "dummydest":
@@ -109,7 +111,7 @@ def _create_instance_dict(**kwargs):
     inst['image_ref'] = 'cedef40a-ed67-4d10-800e-17455edce175'
     inst['reservation_id'] = 'r-fakeres'
     inst['user_id'] = kwargs.get('user_id', 'admin')
-    inst['project_id'] = kwargs.get('project_id', 'fake')
+    inst['project_id'] = kwargs.get('project_id', fakes.FAKE_PROJECT_ID)
     inst['instance_type_id'] = '1'
     if 'host' in kwargs:
         inst['host'] = kwargs.get('host')
@@ -126,38 +128,33 @@ def _create_instance_dict(**kwargs):
     return inst
 
 
-class FakeRequestWithNovaZone(object):
-    environ = {"nova.context": context_maker.get_admin_context()}
-    GET = {"zone": "nova"}
-
-
 class HostTestCaseV21(test.TestCase):
     """Test Case for hosts."""
     validation_ex = exception.ValidationError
     Controller = os_hosts_v21.HostController
-    policy_ex = exception.PolicyNotAuthorized
 
     def _setup_stubs(self):
         # Pretend we have fake_hosts.HOST_LIST in the DB
-        self.stub_out('nova.db.service_get_all',
+        self.stub_out('nova.db.api.service_get_all',
                       stub_service_get_all)
         # Only hosts in our fake DB exist
-        self.stub_out('nova.db.service_get_by_host_and_binary',
+        self.stub_out('nova.db.api.service_get_by_host_and_binary',
                       stub_service_get_by_host_and_binary)
         # 'host_c1' always succeeds, and 'host_c2'
-        self.stubs.Set(self.hosts_api, 'set_host_enabled',
-                       stub_set_host_enabled)
+        self.stub_out('nova.compute.api.HostAPI.set_host_enabled',
+                      stub_set_host_enabled)
         # 'host_c1' always succeeds, and 'host_c2'
-        self.stubs.Set(self.hosts_api, 'set_host_maintenance',
-                       stub_set_host_maintenance)
-        self.stubs.Set(self.hosts_api, 'host_power_action',
-                       stub_host_power_action)
+        self.stub_out('nova.compute.api.HostAPI.set_host_maintenance',
+                      stub_set_host_maintenance)
+        self.stub_out('nova.compute.api.HostAPI.host_power_action',
+                      stub_host_power_action)
 
     def setUp(self):
         super(HostTestCaseV21, self).setUp()
         self.controller = self.Controller()
         self.hosts_api = self.controller.api
         self.req = fakes.HTTPRequest.blank('', use_admin_context=True)
+        self.useFixture(fixtures.SingleCellSimple())
 
         self._setup_stubs()
 
@@ -173,6 +170,49 @@ class HostTestCaseV21(test.TestCase):
         hosts = result['hosts']
         self.assertEqual(fake_hosts.HOST_LIST, hosts)
 
+    def test_list_host_with_multi_filter(self):
+        query_string = 'zone=nova1&zone=nova'
+        req = fakes.HTTPRequest.blank('', use_admin_context=True,
+                                      query_string=query_string)
+        result = self.controller.index(req)
+        self.assertIn('hosts', result)
+        hosts = result['hosts']
+        self.assertEqual(fake_hosts.HOST_LIST_NOVA_ZONE, hosts)
+
+    def test_list_host_query_allow_negative_int_as_string(self):
+        req = fakes.HTTPRequest.blank('', use_admin_context=True,
+                                      query_string='zone=-1')
+        result = self.controller.index(req)
+        self.assertIn('hosts', result)
+        hosts = result['hosts']
+        self.assertEqual([], hosts)
+
+    def test_list_host_query_allow_int_as_string(self):
+        req = fakes.HTTPRequest.blank('', use_admin_context=True,
+                                      query_string='zone=123')
+        result = self.controller.index(req)
+        self.assertIn('hosts', result)
+        hosts = result['hosts']
+        self.assertEqual([], hosts)
+
+    def test_list_host_with_unknown_filter(self):
+        query_string = 'unknown_filter=abc'
+        req = fakes.HTTPRequest.blank('', use_admin_context=True,
+                                      query_string=query_string)
+        result = self.controller.index(req)
+        self.assertIn('hosts', result)
+        hosts = result['hosts']
+        self.assertEqual(fake_hosts.HOST_LIST, hosts)
+
+    def test_list_host_with_hypervisor_and_additional_filter(self):
+        query_string = 'zone=nova&additional_filter=nova2'
+        req = fakes.HTTPRequest.blank('', use_admin_context=True,
+                                      query_string=query_string)
+        result = self.controller.index(req)
+        self.assertIn('hosts', result)
+        hosts = result['hosts']
+        self.assertEqual(fake_hosts.HOST_LIST_NOVA_ZONE, hosts)
+
     def test_disable_host(self):
         self._test_host_update('host_c1', 'status', 'disable', 'disabled')
         self._test_host_update('host_c2', 'status', 'disable', 'enabled')
@@ -193,7 +233,7 @@ class HostTestCaseV21(test.TestCase):
         def stub_service_get_all_notimpl(self, req):
             return [{'host': 'notimplemented', 'topic': None,
                      'availability_zone': None}]
-        self.stub_out('nova.db.service_get_all',
+        self.stub_out('nova.db.api.service_get_all',
                       stub_service_get_all_notimpl)
         body = {key: val}
         self.assertRaises(webob.exc.HTTPNotImplemented,
@@ -369,36 +409,42 @@ class HostTestCaseV21(test.TestCase):
         db.instance_destroy(ctxt, i_ref1['uuid'])
         db.instance_destroy(ctxt, i_ref2['uuid'])
 
+    def test_show_late_host_mapping_gone(self):
+        s_ref = self._create_compute_service()
+        with mock.patch.object(self.controller.api,
+                               'instance_get_all_by_host') as m:
+            m.side_effect = exception.HostMappingNotFound(name='something')
+            self.assertRaises(webob.exc.HTTPNotFound,
+                              self.controller.show, self.req, s_ref['host'])
+
     def test_list_hosts_with_zone(self):
-        result = self.controller.index(FakeRequestWithNovaZone())
+        query_string = 'zone=nova'
+        req = fakes.HTTPRequest.blank('', use_admin_context=True,
+                                      query_string=query_string)
+        result = self.controller.index(req)
         self.assertIn('hosts', result)
         hosts = result['hosts']
         self.assertEqual(fake_hosts.HOST_LIST_NOVA_ZONE, hosts)
 
 
-class HostsPolicyEnforcementV21(test.NoDBTestCase):
+class HostControllerDeprecationTest(test.NoDBTestCase):
 
     def setUp(self):
-        super(HostsPolicyEnforcementV21, self).setUp()
+        super(HostControllerDeprecationTest, self).setUp()
         self.controller = os_hosts_v21.HostController()
-        self.req = fakes.HTTPRequest.blank('')
+        self.req = fakes.HTTPRequest.blank('', version='2.43')
 
-    def test_index_policy_failed(self):
-        rule_name = "os_compute_api:os-hosts"
-        self.policy.set_rules({rule_name: "project_id:non_fake"})
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.index, self.req)
-        self.assertEqual(
-            "Policy doesn't allow %s to be performed." % rule_name,
-            exc.format_message())
-
-    def test_show_policy_failed(self):
-        rule_name = "os_compute_api:os-hosts"
-        self.policy.set_rules({rule_name: "project_id:non_fake"})
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.show, self.req, 1)
-        self.assertEqual(
-            "Policy doesn't allow %s to be performed." % rule_name,
-            exc.format_message())
+    def test_not_found_for_all_host_api(self):
+        self.assertRaises(exception.VersionNotFoundForAPIMethod,
+                          self.controller.show, self.req, fakes.FAKE_UUID)
+        self.assertRaises(exception.VersionNotFoundForAPIMethod,
+                          self.controller.startup, self.req, fakes.FAKE_UUID)
+        self.assertRaises(exception.VersionNotFoundForAPIMethod,
+                          self.controller.shutdown, self.req, fakes.FAKE_UUID)
+        self.assertRaises(exception.VersionNotFoundForAPIMethod,
+                          self.controller.reboot, self.req, fakes.FAKE_UUID)
+        self.assertRaises(exception.VersionNotFoundForAPIMethod,
+                          self.controller.index, self.req)
+        self.assertRaises(exception.VersionNotFoundForAPIMethod,
+                          self.controller.update, self.req, fakes.FAKE_UUID,
+                          body={})
