@@ -15,9 +15,11 @@
 #    under the License.
 
 from oslo_config import cfg
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova import exception
 from nova.network import model
+from nova import objects
 from nova import test
 from nova.tests.unit import fake_network_cache_model
 from nova.virt import netutils
@@ -26,7 +28,6 @@ from nova.virt import netutils
 class RouteTests(test.NoDBTestCase):
     def test_create_route_with_attrs(self):
         route = fake_network_cache_model.new_route()
-        fake_network_cache_model.new_ip(dict(address='192.168.1.1'))
         self.assertEqual('0.0.0.0/24', route['cidr'])
         self.assertEqual('192.168.1.1', route['gateway']['address'])
         self.assertEqual('eth0', route['interface'])
@@ -276,7 +277,7 @@ class SubnetTests(test.NoDBTestCase):
 class NetworkTests(test.NoDBTestCase):
     def test_create_network(self):
         network = fake_network_cache_model.new_network()
-        self.assertEqual(1, network['id'])
+        self.assertEqual(uuids.network_id, network['id'])
         self.assertEqual('br0', network['bridge'])
         self.assertEqual('public', network['label'])
         self.assertEqual(
@@ -330,11 +331,9 @@ class NetworkTests(test.NoDBTestCase):
         self.assertNotEqual(network1, network2)
 
     def test_hydrate(self):
-        fake_network_cache_model.new_subnet()
-        fake_network_cache_model.new_subnet(dict(cidr='255.255.255.255'))
         network = model.Network.hydrate(fake_network_cache_model.new_network())
 
-        self.assertEqual(1, network['id'])
+        self.assertEqual(uuids.network_id, network['id'])
         self.assertEqual('br0', network['bridge'])
         self.assertEqual('public', network['label'])
         self.assertEqual(
@@ -346,7 +345,7 @@ class NetworkTests(test.NoDBTestCase):
 class VIFTests(test.NoDBTestCase):
     def test_create_vif(self):
         vif = fake_network_cache_model.new_vif()
-        self.assertEqual(1, vif['id'])
+        self.assertEqual(uuids.vif_id, vif['id'])
         self.assertEqual('aa:aa:aa:aa:aa:aa', vif['address'])
         self.assertEqual(fake_network_cache_model.new_network(),
                          vif['network'])
@@ -403,12 +402,12 @@ class VIFTests(test.NoDBTestCase):
 
     def test_create_vif_with_type(self):
         vif_dict = dict(
-            id=1,
+            id=uuids.vif_id,
             address='aa:aa:aa:aa:aa:aa',
             network=fake_network_cache_model.new_network(),
             type='bridge')
         vif = fake_network_cache_model.new_vif(vif_dict)
-        self.assertEqual(1, vif['id'])
+        self.assertEqual(uuids.vif_id, vif['id'])
         self.assertEqual('aa:aa:aa:aa:aa:aa', vif['address'])
         self.assertEqual('bridge', vif['type'])
         self.assertEqual(fake_network_cache_model.new_network(),
@@ -423,6 +422,11 @@ class VIFTests(test.NoDBTestCase):
         ] * 2
         self.assertEqual(fixed_ips, ips)
 
+    def test_vif_get_fixed_ips_network_is_none(self):
+        vif = model.VIF()
+        fixed_ips = vif.fixed_ips()
+        self.assertEqual([], fixed_ips)
+
     def test_vif_get_floating_ips(self):
         vif = fake_network_cache_model.new_vif()
         vif['network']['subnets'][0]['ips'][0].add_floating_ip('192.168.1.1')
@@ -433,7 +437,7 @@ class VIFTests(test.NoDBTestCase):
         vif = fake_network_cache_model.new_vif()
         labeled_ips = vif.labeled_ips()
         ip_dict = {
-            'network_id': 1,
+            'network_id': uuids.network_id,
             'ips': [fake_network_cache_model.new_ip(
                         {'address': '10.10.0.2', 'type': 'fixed'}),
                     fake_network_cache_model.new_ip(
@@ -442,21 +446,20 @@ class VIFTests(test.NoDBTestCase):
         self.assertEqual(ip_dict, labeled_ips)
 
     def test_hydrate(self):
-        fake_network_cache_model.new_network()
         vif = model.VIF.hydrate(fake_network_cache_model.new_vif())
-        self.assertEqual(1, vif['id'])
+        self.assertEqual(uuids.vif_id, vif['id'])
         self.assertEqual('aa:aa:aa:aa:aa:aa', vif['address'])
         self.assertEqual(fake_network_cache_model.new_network(),
                          vif['network'])
 
     def test_hydrate_vif_with_type(self):
         vif_dict = dict(
-            id=1,
+            id=uuids.vif_id,
             address='aa:aa:aa:aa:aa:aa',
             network=fake_network_cache_model.new_network(),
             type='bridge')
         vif = model.VIF.hydrate(fake_network_cache_model.new_vif(vif_dict))
-        self.assertEqual(1, vif['id'])
+        self.assertEqual(uuids.vif_id, vif['id'])
         self.assertEqual('aa:aa:aa:aa:aa:aa', vif['address'])
         self.assertEqual('bridge', vif['type'])
         self.assertEqual(fake_network_cache_model.new_network(),
@@ -573,7 +576,7 @@ class NetworkInfoTests(test.NoDBTestCase):
 
         nwinfo = model.NetworkInfo(vifs)
         return netutils.get_injected_network_template(
-            nwinfo, use_ipv6=use_ipv6, libvirt_virt_type=libvirt_virt_type)
+            nwinfo, libvirt_virt_type=libvirt_virt_type)
 
     def test_injection_dynamic(self):
         expected = None
@@ -647,7 +650,7 @@ iface eth0 inet static
         template = self._setup_injected_network_scenario(dns=False)
         self.assertEqual(expected, template)
 
-    def test_injection_static_overriden_template(self):
+    def test_injection_static_overridden_template(self):
         cfg.CONF.set_override(
             'injected_network_template',
             'nova/tests/unit/network/interfaces-override.template')
@@ -852,25 +855,78 @@ iface eth1 inet static
                 libvirt_virt_type='lxc')
         self.assertEqual(expected, template)
 
+    def test_get_events(self):
+        network_info = model.NetworkInfo([
+            model.VIF(
+                id=uuids.hybrid_vif,
+                details={'ovs_hybrid_plug': True}),
+            model.VIF(
+                id=uuids.normal_vif,
+                details={'ovs_hybrid_plug': False})])
+        same_host = objects.Migration(source_compute='fake-host',
+                                      dest_compute='fake-host')
+        diff_host = objects.Migration(source_compute='fake-host1',
+                                      dest_compute='fake-host2')
+        # Same-host migrations will have all events be plug-time.
+        self.assertCountEqual(
+            [('network-vif-plugged', uuids.normal_vif),
+             ('network-vif-plugged', uuids.hybrid_vif)],
+            network_info.get_plug_time_events(same_host))
+        # Same host migration will have no plug-time events.
+        self.assertEqual([], network_info.get_bind_time_events(same_host))
+        # Diff-host migration + OVS hybrid plug = bind-time events
+        self.assertEqual(
+            [('network-vif-plugged', uuids.hybrid_vif)],
+            network_info.get_bind_time_events(diff_host))
+        # Diff-host migration + normal OVS = plug-time events
+        self.assertEqual(
+            [('network-vif-plugged', uuids.normal_vif)],
+            network_info.get_plug_time_events(diff_host))
+
+    def test_has_port_with_allocation(self):
+        network_info = model.NetworkInfo([])
+        self.assertFalse(network_info.has_port_with_allocation())
+
+        network_info.append(
+            model.VIF(id=uuids.port_without_profile))
+        self.assertFalse(network_info.has_port_with_allocation())
+
+        network_info.append(
+            model.VIF(id=uuids.port_no_allocation, profile={'foo': 'bar'}))
+        self.assertFalse(network_info.has_port_with_allocation())
+
+        network_info.append(
+            model.VIF(
+                id=uuids.port_empty_alloc, profile={'allocation': None}))
+        self.assertFalse(network_info.has_port_with_allocation())
+
+        network_info.append(
+            model.VIF(
+                id=uuids.port_with_alloc, profile={'allocation': uuids.rp}))
+        self.assertTrue(network_info.has_port_with_allocation())
+
 
 class TestNetworkMetadata(test.NoDBTestCase):
     def setUp(self):
         super(TestNetworkMetadata, self).setUp()
-        self.netinfo = model.NetworkInfo([fake_network_cache_model.new_vif(
-            {'type': 'ethernet'})])
+        self.netinfo = self._new_netinfo()
+
+    def _new_netinfo(self, vif_type='ethernet'):
+        netinfo = model.NetworkInfo([fake_network_cache_model.new_vif(
+            {'type': vif_type})])
 
         # Give this vif ipv4 and ipv6 dhcp subnets
         ipv4_subnet = fake_network_cache_model.new_subnet(version=4)
         ipv6_subnet = fake_network_cache_model.new_subnet(version=6)
 
-        self.netinfo[0]['network']['subnets'][0] = ipv4_subnet
-        self.netinfo[0]['network']['subnets'][1] = ipv6_subnet
-        self.netinfo[0]['network']['meta']['mtu'] = 1500
+        netinfo[0]['network']['subnets'][0] = ipv4_subnet
+        netinfo[0]['network']['subnets'][1] = ipv6_subnet
+        netinfo[0]['network']['meta']['mtu'] = 1500
+        return netinfo
 
     def test_get_network_metadata_json(self):
 
-        net_metadata = netutils.get_network_metadata(self.netinfo,
-                                                     use_ipv6=True)
+        net_metadata = netutils.get_network_metadata(self.netinfo)
 
         # Physical Ethernet
         self.assertEqual(
@@ -878,7 +934,7 @@ class TestNetworkMetadata(test.NoDBTestCase):
                 'id': 'interface0',
                 'type': 'phy',
                 'ethernet_mac_address': 'aa:aa:aa:aa:aa:aa',
-                'vif_id': 1,
+                'vif_id': uuids.vif_id,
                 'mtu': 1500
             },
             net_metadata['links'][0])
@@ -903,7 +959,9 @@ class TestNetworkMetadata(test.NoDBTestCase):
                         'gateway': '192.168.1.1'
                     }
                 ],
-                'network_id': 1
+                'services': [{'address': '1.2.3.4', 'type': 'dns'},
+                             {'address': '2.3.4.5', 'type': 'dns'}],
+                'network_id': uuids.network_id
             },
             net_metadata['networks'][0])
 
@@ -926,7 +984,9 @@ class TestNetworkMetadata(test.NoDBTestCase):
                         'gateway': 'fd00::1:1'
                     }
                 ],
-                'network_id': 1
+                'services': [{'address': '1:2:3:4::', 'type': 'dns'},
+                             {'address': '2:3:4:5::', 'type': 'dns'}],
+                'network_id': uuids.network_id
             },
             net_metadata['networks'][1])
 
@@ -938,8 +998,7 @@ class TestNetworkMetadata(test.NoDBTestCase):
 
         self.netinfo[0]['network']['subnets'][0] = ipv4_subnet
         self.netinfo[0]['network']['subnets'][1] = ipv6_subnet
-        net_metadata = netutils.get_network_metadata(self.netinfo,
-                                                     use_ipv6=True)
+        net_metadata = netutils.get_network_metadata(self.netinfo)
 
         # IPv4 Network
         self.assertEqual(
@@ -947,7 +1006,7 @@ class TestNetworkMetadata(test.NoDBTestCase):
                 'id': 'network0',
                 'link': 'interface0',
                 'type': 'ipv4_dhcp',
-                'network_id': 1
+                'network_id': uuids.network_id
             },
             net_metadata['networks'][0])
 
@@ -957,17 +1016,61 @@ class TestNetworkMetadata(test.NoDBTestCase):
                 'id': 'network1',
                 'link': 'interface0',
                 'type': 'ipv6_dhcp',
-                'network_id': 1
+                'network_id': uuids.network_id
             },
             net_metadata['networks'][1])
 
+    def _test_get_network_metadata_json_ipv6_addr_mode(self, mode):
+        ipv6_subnet = fake_network_cache_model.new_subnet(
+            subnet_dict=dict(dhcp_server='1234:567::',
+                             ipv6_address_mode=mode), version=6)
+
+        self.netinfo[0]['network']['subnets'][1] = ipv6_subnet
+        net_metadata = netutils.get_network_metadata(self.netinfo)
+
+        self.assertEqual(
+            {
+                'id': 'network1',
+                'link': 'interface0',
+                'ip_address': 'fd00::2',
+                'netmask': 'ffff:ffff:ffff::',
+                'routes': [
+                    {
+                        'network': '::',
+                        'netmask': '::',
+                        'gateway': 'fd00::1'
+                    },
+                    {
+                        'network': '::',
+                        'netmask': 'ffff:ffff:ffff::',
+                        'gateway': 'fd00::1:1'
+                    }
+                ],
+                'services': [
+                    {'address': '1:2:3:4::', 'type': 'dns'},
+                    {'address': '2:3:4:5::', 'type': 'dns'}
+                ],
+                'type': 'ipv6_%s' % mode,
+                'network_id': uuids.network_id
+            },
+            net_metadata['networks'][1])
+
+    def test_get_network_metadata_json_ipv6_addr_mode_slaac(self):
+        self._test_get_network_metadata_json_ipv6_addr_mode('slaac')
+
+    def test_get_network_metadata_json_ipv6_addr_mode_stateful(self):
+        self._test_get_network_metadata_json_ipv6_addr_mode('dhcpv6-stateful')
+
+    def test_get_network_metadata_json_ipv6_addr_mode_stateless(self):
+        self._test_get_network_metadata_json_ipv6_addr_mode('dhcpv6-stateless')
+
     def test__get_nets(self):
         expected_net = {
             'id': 'network0',
             'ip_address': '10.10.0.2',
             'link': 1,
             'netmask': '255.255.255.0',
-            'network_id': 1,
+            'network_id': uuids.network_id,
             'routes': [
                 {
                     'gateway': '10.10.0.1',
@@ -977,6 +1080,10 @@ class TestNetworkMetadata(test.NoDBTestCase):
                     'gateway': '192.168.1.1',
                     'netmask': '255.255.255.0',
                     'network': '0.0.0.0'}],
+                'services': [
+                    {'address': '1.2.3.4', 'type': 'dns'},
+                    {'address': '2.3.4.5', 'type': 'dns'}
+                ],
             'type': 'ipv4'
         }
         net = netutils._get_nets(
@@ -986,7 +1093,7 @@ class TestNetworkMetadata(test.NoDBTestCase):
     def test__get_eth_link(self):
         expected_link = {
             'id': 'interface0',
-            'vif_id': 1,
+            'vif_id': uuids.vif_id,
             'type': 'vif',
             'ethernet_mac_address': 'aa:aa:aa:aa:aa:aa',
             'mtu': 1500
@@ -998,7 +1105,7 @@ class TestNetworkMetadata(test.NoDBTestCase):
     def test__get_eth_link_physical(self):
         expected_link = {
             'id': 'interface1',
-            'vif_id': 1,
+            'vif_id': uuids.vif_id,
             'type': 'phy',
             'ethernet_mac_address': 'aa:aa:aa:aa:aa:aa',
             'mtu': 1500
@@ -1050,14 +1157,14 @@ class TestNetworkMetadata(test.NoDBTestCase):
                     "ethernet_mac_address": "aa:aa:aa:aa:aa:aa",
                     "id": "interface0",
                     "type": "phy",
-                    "vif_id": 1,
+                    "vif_id": uuids.vif_id,
                     "mtu": 1500
                 },
                 {
                     "ethernet_mac_address": "aa:aa:aa:aa:aa:ab",
                     "id": "interface1",
                     "type": "phy",
-                    "vif_id": 1,
+                    "vif_id": uuids.vif_id,
                     "mtu": 1500
                 },
             ],
@@ -1081,6 +1188,8 @@ class TestNetworkMetadata(test.NoDBTestCase):
                             "network": "0.0.0.0"
                         }
                     ],
+                   'services': [{'address': '1.2.3.4', 'type': 'dns'},
+                                {'address': '2.3.4.5', 'type': 'dns'}],
                     "type": "ipv4"
                 },
                 {
@@ -1095,6 +1204,8 @@ class TestNetworkMetadata(test.NoDBTestCase):
                               {'gateway': 'fd00::1:1',
                                'netmask': 'ffff:ffff:ffff::',
                                'network': '::'}],
+                   'services': [{'address': '1:2:3:4::', 'type': 'dns'},
+                                {'address': '2:3:4:5::', 'type': 'dns'}],
                    'type': 'ipv6'
                 },
                 {
@@ -1111,6 +1222,8 @@ class TestNetworkMetadata(test.NoDBTestCase):
                             "network": "0.0.0.0"
                         }
                     ],
+                   'services': [{'address': '1.2.3.4', 'type': 'dns'},
+                                {'address': '2.3.4.5', 'type': 'dns'}],
                     "type": "ipv4"
                 }
             ],
@@ -1160,7 +1273,7 @@ class TestNetworkMetadata(test.NoDBTestCase):
             ],
             "networks": [
                 {
-                    "network_id": 1,
+                    "network_id": uuids.network_id,
                     "type": "ipv6",
                     "netmask": "ffff:ffff:ffff::",
                     "link": "interface0",
@@ -1176,6 +1289,8 @@ class TestNetworkMetadata(test.NoDBTestCase):
                             "gateway": "fd00::1:1"
                         }
                     ],
+                   'services': [{'address': '1:2:3:4::', 'type': 'dns'},
+                                {'address': '2:3:4:5::', 'type': 'dns'}],
                     "ip_address": "fd00::2",
                     "id": "network0"
                 }
@@ -1186,7 +1301,7 @@ class TestNetworkMetadata(test.NoDBTestCase):
                     "mtu": 1500,
                     "type": "phy",
                     "id": "interface0",
-                    "vif_id": 1
+                    "vif_id": uuids.vif_id
                 }
             ]
         }
@@ -1195,3 +1310,32 @@ class TestNetworkMetadata(test.NoDBTestCase):
         self.netinfo[0]['network']['subnets'].pop(0)
         network_json = netutils.get_network_metadata(self.netinfo)
         self.assertEqual(expected_json, network_json)
+
+    def test_legacy_vif_types_type_passed_through(self):
+        legacy_types = [
+            model.VIF_TYPE_BRIDGE,
+            model.VIF_TYPE_DVS,
+            model.VIF_TYPE_HW_VEB,
+            model.VIF_TYPE_HYPERV,
+            model.VIF_TYPE_OVS,
+            model.VIF_TYPE_TAP,
+            model.VIF_TYPE_VHOSTUSER,
+            model.VIF_TYPE_VIF,
+        ]
+        link_types = []
+        for vif_type in legacy_types:
+            network_json = netutils.get_network_metadata(
+                self._new_netinfo(vif_type=vif_type))
+            link_types.append(network_json["links"][0]["type"])
+
+        self.assertEqual(legacy_types, link_types)
+
+    def test_new_vif_types_get_type_phy(self):
+        new_types = ["whizbang_nvf", "vswitch9"]
+        link_types = []
+        for vif_type in new_types:
+            network_json = netutils.get_network_metadata(
+                self._new_netinfo(vif_type=vif_type))
+            link_types.append(network_json["links"][0]["type"])
+
+        self.assertEqual(["phy"] * len(new_types), link_types)
