@@ -13,22 +13,25 @@
 #    under the License.
 
 import mock
+from oslo_utils.fixture import uuidsentinel
 from oslo_utils import timeutils
 from oslo_versionedobjects import base as ovo_base
 from oslo_versionedobjects import exception as ovo_exc
+import six
 
 from nova.compute import manager as compute_manager
 from nova import context
-from nova import db
+from nova.db import api as db
 from nova import exception
 from nova import objects
 from nova.objects import aggregate
-from nova.objects import fields
 from nova.objects import service
 from nova import test
+from nova.tests import fixtures
 from nova.tests.unit.objects import test_compute_node
 from nova.tests.unit.objects import test_objects
 
+
 NOW = timeutils.utcnow().replace(microsecond=0)
 
 
@@ -39,8 +42,9 @@ def _fake_service(**kwargs):
         'deleted_at': None,
         'deleted': False,
         'id': 123,
+        'uuid': uuidsentinel.service,
         'host': 'fake-host',
-        'binary': 'nova-fake',
+        'binary': 'nova-compute',
         'topic': 'fake-service-topic',
         'report_count': 1,
         'forced_down': False,
@@ -52,6 +56,7 @@ def _fake_service(**kwargs):
     fake_service.update(kwargs)
     return fake_service
 
+
 fake_service = _fake_service()
 
 OPTIONAL = ['availability_zone', 'compute_node']
@@ -77,25 +82,31 @@ class _TestServiceObject(object):
                 'pci_device_pools': 'pci_stats'}
 
     def _test_query(self, db_method, obj_method, *args, **kwargs):
-        self.mox.StubOutWithMock(db, db_method)
         db_exception = kwargs.pop('db_exception', None)
         if db_exception:
-            getattr(db, db_method)(self.context, *args, **kwargs).AndRaise(
-                db_exception)
+            with mock.patch.object(db, db_method, side_effect=db_exception) \
+                    as mock_db_method:
+                obj = getattr(service.Service, obj_method)(self.context, *args,
+                                                           **kwargs)
+                self.assertIsNone(obj)
+                mock_db_method.assert_called_once_with(self.context, *args,
+                                                       **kwargs)
         else:
-            getattr(db, db_method)(self.context, *args, **kwargs).AndReturn(
-                fake_service)
-        self.mox.ReplayAll()
-        obj = getattr(service.Service, obj_method)(self.context, *args,
-                                                   **kwargs)
-        if db_exception:
-            self.assertIsNone(obj)
-        else:
-            self.compare_obj(obj, fake_service, allow_missing=OPTIONAL)
+            with mock.patch.object(db, db_method, return_value=fake_service) \
+                    as mock_db_method:
+                obj = getattr(service.Service, obj_method)(self.context, *args,
+                                                           **kwargs)
+                self.compare_obj(obj, fake_service, allow_missing=OPTIONAL)
+                mock_db_method.assert_called_once_with(self.context, *args,
+                                                       **kwargs)
 
     def test_get_by_id(self):
         self._test_query('service_get', 'get_by_id', 123)
 
+    def test_get_by_uuid(self):
+        self._test_query('service_get_by_uuid', 'get_by_uuid',
+                         uuidsentinel.service_uuid)
+
     def test_get_by_host_and_topic(self):
         self._test_query('service_get_by_host_and_topic',
                          'get_by_host_and_topic', 'fake-host', 'fake-topic')
@@ -118,41 +129,49 @@ class _TestServiceObject(object):
         self._test_query('service_get_by_host_and_binary', 'get_by_args',
                          'fake-host', 'fake-binary')
 
-    def test_create(self):
-        self.mox.StubOutWithMock(db, 'service_create')
-        db.service_create(self.context, {'host': 'fake-host',
-                                         'version': fake_service['version']}
-                          ).AndReturn(fake_service)
-        self.mox.ReplayAll()
+    @mock.patch.object(db, 'service_create', return_value=fake_service)
+    def test_create(self, mock_service_create):
         service_obj = service.Service(context=self.context)
         service_obj.host = 'fake-host'
+        service_obj.uuid = uuidsentinel.service2
         service_obj.create()
         self.assertEqual(fake_service['id'], service_obj.id)
         self.assertEqual(service.SERVICE_VERSION, service_obj.version)
+        mock_service_create.assert_called_once_with(
+                       self.context, {'host': 'fake-host',
+                                      'uuid': uuidsentinel.service2,
+                                      'version': fake_service['version']})
 
-    def test_recreate_fails(self):
-        self.mox.StubOutWithMock(db, 'service_create')
-        db.service_create(self.context, {'host': 'fake-host',
-                                         'version': fake_service['version']}
-                          ).AndReturn(fake_service)
-        self.mox.ReplayAll()
+    @mock.patch('nova.objects.service.uuidutils.generate_uuid',
+                return_value=uuidsentinel.service3)
+    @mock.patch.object(db, 'service_create', return_value=fake_service)
+    def test_create_without_uuid_generates_one(
+            self, mock_service_create, generate_uuid):
+        service_obj = service.Service(context=self.context)
+        service_obj.create()
+        create_args = mock_service_create.call_args[0][1]
+        self.assertEqual(generate_uuid.return_value, create_args['uuid'])
+
+    @mock.patch.object(db, 'service_create', return_value=fake_service)
+    def test_recreate_fails(self, mock_service_create):
         service_obj = service.Service(context=self.context)
         service_obj.host = 'fake-host'
         service_obj.create()
         self.assertRaises(exception.ObjectActionError, service_obj.create)
+        mock_service_create(self.context, {'host': 'fake-host',
+                                         'version': fake_service['version']})
 
-    def test_save(self):
-        self.mox.StubOutWithMock(db, 'service_update')
-        db.service_update(self.context, 123,
-                          {'host': 'fake-host',
-                           'version': fake_service['version']}
-                          ).AndReturn(fake_service)
-        self.mox.ReplayAll()
+    @mock.patch('nova.objects.Service._send_notification')
+    @mock.patch.object(db, 'service_update', return_value=fake_service)
+    def test_save(self, mock_service_update, mock_notify):
         service_obj = service.Service(context=self.context)
         service_obj.id = 123
         service_obj.host = 'fake-host'
         service_obj.save()
         self.assertEqual(service.SERVICE_VERSION, service_obj.version)
+        mock_service_update.assert_called_once_with(
+            self.context, 123, {'host': 'fake-host',
+                                'version': fake_service['version']})
 
     @mock.patch.object(db, 'service_create',
                        return_value=fake_service)
@@ -163,13 +182,13 @@ class _TestServiceObject(object):
         self.assertRaises(ovo_exc.ReadOnlyFieldError, setattr,
                           service_obj, 'id', 124)
 
-    def _test_destroy(self):
-        self.mox.StubOutWithMock(db, 'service_destroy')
-        db.service_destroy(self.context, 123)
-        self.mox.ReplayAll()
+    @mock.patch('nova.objects.Service._send_notification')
+    @mock.patch.object(db, 'service_destroy')
+    def _test_destroy(self, mock_service_destroy, mock_notify):
         service_obj = service.Service(context=self.context)
         service_obj.id = 123
         service_obj.destroy()
+        mock_service_destroy.assert_called_once_with(self.context, 123)
 
     def test_destroy(self):
         # The test harness needs db.service_destroy to work,
@@ -180,16 +199,15 @@ class _TestServiceObject(object):
         finally:
             db.service_destroy = orig_service_destroy
 
-    def test_get_by_topic(self):
-        self.mox.StubOutWithMock(db, 'service_get_all_by_topic')
-        db.service_get_all_by_topic(self.context, 'fake-topic').AndReturn(
-            [fake_service])
-        self.mox.ReplayAll()
+    @mock.patch.object(db, 'service_get_all_by_topic',
+                       return_value=[fake_service])
+    def test_get_by_topic(self, mock_service_get):
         services = service.ServiceList.get_by_topic(self.context, 'fake-topic')
         self.assertEqual(1, len(services))
         self.compare_obj(services[0], fake_service, allow_missing=OPTIONAL)
+        mock_service_get.assert_called_once_with(self.context, 'fake-topic')
 
-    @mock.patch('nova.db.service_get_all_by_binary')
+    @mock.patch('nova.db.api.service_get_all_by_binary')
     def test_get_by_binary(self, mock_get):
         mock_get.return_value = [fake_service]
         services = service.ServiceList.get_by_binary(self.context,
@@ -199,7 +217,7 @@ class _TestServiceObject(object):
                                          'fake-binary',
                                          include_disabled=False)
 
-    @mock.patch('nova.db.service_get_all_by_binary')
+    @mock.patch('nova.db.api.service_get_all_by_binary')
     def test_get_by_binary_disabled(self, mock_get):
         mock_get.return_value = [_fake_service(disabled=True)]
         services = service.ServiceList.get_by_binary(self.context,
@@ -210,7 +228,7 @@ class _TestServiceObject(object):
                                          'fake-binary',
                                          include_disabled=True)
 
-    @mock.patch('nova.db.service_get_all_by_binary')
+    @mock.patch('nova.db.api.service_get_all_by_binary')
     def test_get_by_binary_both(self, mock_get):
         mock_get.return_value = [_fake_service(),
                                  _fake_service(disabled=True)]
@@ -222,51 +240,44 @@ class _TestServiceObject(object):
                                          'fake-binary',
                                          include_disabled=True)
 
-    def test_get_by_host(self):
-        self.mox.StubOutWithMock(db, 'service_get_all_by_host')
-        db.service_get_all_by_host(self.context, 'fake-host').AndReturn(
-            [fake_service])
-        self.mox.ReplayAll()
+    @mock.patch.object(db, 'service_get_all_by_host',
+                       return_value=[fake_service])
+    def test_get_by_host(self, mock_service_get):
         services = service.ServiceList.get_by_host(self.context, 'fake-host')
         self.assertEqual(1, len(services))
         self.compare_obj(services[0], fake_service, allow_missing=OPTIONAL)
+        mock_service_get.assert_called_once_with(self.context, 'fake-host')
 
-    def test_get_all(self):
-        self.mox.StubOutWithMock(db, 'service_get_all')
-        db.service_get_all(self.context, disabled=False).AndReturn(
-            [fake_service])
-        self.mox.ReplayAll()
+    @mock.patch.object(db, 'service_get_all', return_value=[fake_service])
+    def test_get_all(self, mock_get_all):
         services = service.ServiceList.get_all(self.context, disabled=False)
         self.assertEqual(1, len(services))
         self.compare_obj(services[0], fake_service, allow_missing=OPTIONAL)
+        mock_get_all.assert_called_once_with(self.context, disabled=False)
 
-    def test_get_all_with_az(self):
-        self.mox.StubOutWithMock(db, 'service_get_all')
-        self.mox.StubOutWithMock(aggregate.AggregateList,
-                                 'get_by_metadata_key')
-        db.service_get_all(self.context, disabled=None).AndReturn(
-            [dict(fake_service, topic='compute')])
+    @mock.patch.object(db, 'service_get_all')
+    @mock.patch.object(aggregate.AggregateList, 'get_by_metadata_key')
+    def test_get_all_with_az(self, mock_get_by_key, mock_get_all):
         agg = aggregate.Aggregate(context=self.context)
         agg.name = 'foo'
         agg.metadata = {'availability_zone': 'test-az'}
         agg.create()
         agg.hosts = [fake_service['host']]
-        aggregate.AggregateList.get_by_metadata_key(self.context,
-            'availability_zone', hosts=set(agg.hosts)).AndReturn([agg])
-        self.mox.ReplayAll()
+        mock_get_by_key.return_value = [agg]
+        mock_get_all.return_value = [dict(fake_service, topic='compute')]
         services = service.ServiceList.get_all(self.context, set_zones=True)
         self.assertEqual(1, len(services))
         self.assertEqual('test-az', services[0].availability_zone)
+        mock_get_all.assert_called_once_with(self.context, disabled=None)
+        mock_get_by_key.assert_called_once_with(self.context,
+                         'availability_zone', hosts=set(agg.hosts))
 
-    def test_compute_node(self):
+    @mock.patch.object(objects.ComputeNodeList, 'get_all_by_host')
+    def test_compute_node(self, mock_get):
         fake_compute_node = objects.ComputeNode._from_db_object(
             self.context, objects.ComputeNode(),
             test_compute_node.fake_compute_node)
-        self.mox.StubOutWithMock(objects.ComputeNodeList, 'get_all_by_host')
-        objects.ComputeNodeList.get_all_by_host(
-            self.context, 'fake-host').AndReturn(
-                [fake_compute_node])
-        self.mox.ReplayAll()
+        mock_get.return_value = [fake_compute_node]
         service_obj = service.Service(id=123, host="fake-host",
                                       binary="nova-compute")
         service_obj._context = self.context
@@ -274,6 +285,17 @@ class _TestServiceObject(object):
                          fake_compute_node)
         # Make sure it doesn't re-fetch this
         service_obj.compute_node
+        mock_get.assert_called_once_with(self.context, 'fake-host')
+
+    @mock.patch.object(db, 'service_get_all_computes_by_hv_type')
+    def test_get_all_computes_by_hv_type(self, mock_get_all):
+        mock_get_all.return_value = [fake_service]
+        services = service.ServiceList.get_all_computes_by_hv_type(
+            self.context, 'hv-type')
+        self.assertEqual(1, len(services))
+        self.compare_obj(services[0], fake_service, allow_missing=OPTIONAL)
+        mock_get_all.assert_called_once_with(self.context, 'hv-type',
+                                             include_disabled=False)
 
     def test_load_when_orphaned(self):
         service_obj = service.Service()
@@ -298,23 +320,23 @@ class _TestServiceObject(object):
                                               version_manifest=versions),
             fake_service_dict['compute_node'])
 
-    @mock.patch('nova.db.service_get_minimum_version')
+    @mock.patch('nova.db.api.service_get_minimum_version')
     def test_get_minimum_version_none(self, mock_get):
         mock_get.return_value = None
         self.assertEqual(0,
                          objects.Service.get_minimum_version(self.context,
                                                              'nova-compute'))
-        mock_get.assert_called_once_with(self.context, 'nova-compute')
+        mock_get.assert_called_once_with(self.context, ['nova-compute'])
 
-    @mock.patch('nova.db.service_get_minimum_version')
+    @mock.patch('nova.db.api.service_get_minimum_version')
     def test_get_minimum_version(self, mock_get):
-        mock_get.return_value = 123
+        mock_get.return_value = {'nova-compute': 123}
         self.assertEqual(123,
                          objects.Service.get_minimum_version(self.context,
                                                              'nova-compute'))
-        mock_get.assert_called_once_with(self.context, 'nova-compute')
+        mock_get.assert_called_once_with(self.context, ['nova-compute'])
 
-    @mock.patch('nova.db.service_get_minimum_version')
+    @mock.patch('nova.db.api.service_get_minimum_version')
     @mock.patch('nova.objects.service.LOG')
     def test_get_minimum_version_checks_binary(self, mock_log, mock_get):
         mock_get.return_value = None
@@ -328,10 +350,10 @@ class _TestServiceObject(object):
                           'compute')
         self.assertTrue(mock_log.warning.called)
 
-    @mock.patch('nova.db.service_get_minimum_version')
+    @mock.patch('nova.db.api.service_get_minimum_version')
     def test_get_minimum_version_with_caching(self, mock_get):
         objects.Service.enable_min_version_cache()
-        mock_get.return_value = 123
+        mock_get.return_value = {'nova-compute': 123}
         self.assertEqual(123,
                          objects.Service.get_minimum_version(self.context,
                                                              'nova-compute'))
@@ -340,25 +362,54 @@ class _TestServiceObject(object):
         self.assertEqual(123,
                          objects.Service.get_minimum_version(self.context,
                                                              'nova-compute'))
-        mock_get.assert_called_once_with(self.context, 'nova-compute')
+        mock_get.assert_called_once_with(self.context, ['nova-compute'])
         objects.Service._SERVICE_VERSION_CACHING = False
         objects.Service.clear_min_version_cache()
 
-    @mock.patch('nova.db.service_get_minimum_version', return_value=2)
-    def test_create_above_minimum(self, mock_get):
+    @mock.patch('nova.db.api.service_get_minimum_version')
+    def test_get_min_version_multiple_with_old(self, mock_gmv):
+        mock_gmv.return_value = {'nova-api': None,
+                                 'nova-scheduler': 2,
+                                 'nova-conductor': 3}
+
+        binaries = ['nova-api', 'nova-api', 'nova-conductor',
+                    'nova-conductor', 'nova-api']
+        minimum = objects.Service.get_minimum_version_multi(self.context,
+                                                            binaries)
+        self.assertEqual(0, minimum)
+
+    @mock.patch('nova.db.api.service_get_minimum_version')
+    def test_get_min_version_multiple(self, mock_gmv):
+        mock_gmv.return_value = {'nova-api': 1,
+                                 'nova-scheduler': 2,
+                                 'nova-conductor': 3}
+
+        binaries = ['nova-api', 'nova-api', 'nova-conductor',
+                    'nova-conductor', 'nova-api']
+        minimum = objects.Service.get_minimum_version_multi(self.context,
+                                                            binaries)
+        self.assertEqual(1, minimum)
+
+    @mock.patch('nova.objects.Service._send_notification')
+    @mock.patch('nova.db.api.service_get_minimum_version',
+                return_value={'nova-compute': 2})
+    def test_create_above_minimum(self, mock_get, mock_notify):
         with mock.patch('nova.objects.service.SERVICE_VERSION',
                         new=3):
             objects.Service(context=self.context,
                             binary='nova-compute').create()
 
-    @mock.patch('nova.db.service_get_minimum_version', return_value=2)
-    def test_create_equal_to_minimum(self, mock_get):
+    @mock.patch('nova.objects.Service._send_notification')
+    @mock.patch('nova.db.api.service_get_minimum_version',
+                return_value={'nova-compute': 2})
+    def test_create_equal_to_minimum(self, mock_get, mock_notify):
         with mock.patch('nova.objects.service.SERVICE_VERSION',
                         new=2):
             objects.Service(context=self.context,
                             binary='nova-compute').create()
 
-    @mock.patch('nova.db.service_get_minimum_version', return_value=2)
+    @mock.patch('nova.db.api.service_get_minimum_version',
+                return_value={'nova-compute': 2})
     def test_create_below_minimum(self, mock_get):
         with mock.patch('nova.objects.service.SERVICE_VERSION',
                         new=1):
@@ -367,6 +418,14 @@ class _TestServiceObject(object):
                                               binary='nova-compute',
                                               ).create)
 
+    @mock.patch('nova.objects.base.NovaObject'
+                '.obj_make_compatible_from_manifest', new=mock.Mock())
+    def test_obj_make_compatible_from_manifest_strips_uuid(self):
+        s = service.Service()
+        primitive = {'uuid': uuidsentinel.service}
+        s.obj_make_compatible_from_manifest(primitive, '1.20', mock.Mock())
+        self.assertNotIn('uuid', primitive)
+
 
 class TestServiceObject(test_objects._LocalTest,
                         _TestServiceObject):
@@ -378,7 +437,7 @@ class TestRemoteServiceObject(test_objects._RemoteTest,
     pass
 
 
-class TestServiceVersion(test.TestCase):
+class TestServiceVersion(test.NoDBTestCase):
     def setUp(self):
         self.ctxt = context.get_admin_context()
         super(TestServiceVersion, self).setUp()
@@ -400,7 +459,7 @@ class TestServiceVersion(test.TestCase):
         self.assertEqual(
             current, calculated,
             'Changes detected that require a SERVICE_VERSION change. Please '
-            'increment nova.objects.service.SERVICE_VERSION, and make sure it'
+            'increment nova.objects.service.SERVICE_VERSION, and make sure it '
             'is equal to nova.compute.manager.ComputeManager.target.version.')
 
     def test_version_in_init(self):
@@ -420,71 +479,91 @@ class TestServiceVersion(test.TestCase):
         obj._from_db_object(self.ctxt, obj, fake_different_service)
         self.assertEqual(fake_version, obj.version)
 
-    def test_save_noop_with_only_version(self):
-        o = objects.Service(context=self.ctxt, id=fake_service['id'])
-        o.obj_reset_changes(['id'])
-        self.assertEqual(set(['version']), o.obj_what_changed())
-        with mock.patch('nova.db.service_update') as mock_update:
-            o.save()
-            self.assertFalse(mock_update.called)
-        o.host = 'foo'
-        with mock.patch('nova.db.service_update') as mock_update:
-            mock_update.return_value = fake_service
-            o.save()
-            mock_update.assert_called_once_with(
-                self.ctxt, fake_service['id'],
-                {'version': service.SERVICE_VERSION,
-                 'host': 'foo'})
 
+class TestServiceVersionCells(test.TestCase):
 
-class TestServiceStatusNotification(test.TestCase):
     def setUp(self):
-        self.ctxt = context.get_admin_context()
-        super(TestServiceStatusNotification, self).setUp()
+        self.context = context.get_admin_context()
+        super(TestServiceVersionCells, self).setUp()
 
-    @mock.patch('nova.objects.service.ServiceStatusNotification')
-    def _verify_notification(self, service_obj, mock_notification):
-        service_obj.save()
+    def _setup_cells(self):
+        # NOTE(danms): Override the base class's cell setup so we can have two
+        self.cells = fixtures.CellDatabases()
+        self.cells.add_cell_database(uuidsentinel.cell1, default=True)
+        self.cells.add_cell_database(uuidsentinel.cell2)
+        self.useFixture(self.cells)
 
-        self.assertTrue(mock_notification.called)
+        cm = objects.CellMapping(context=self.context,
+                                 uuid=uuidsentinel.cell1,
+                                 name='cell1',
+                                 transport_url='fake://nowhere/',
+                                 database_connection=uuidsentinel.cell1)
+        cm.create()
+        cm = objects.CellMapping(context=self.context,
+                                 uuid=uuidsentinel.cell2,
+                                 name='cell2',
+                                 transport_url='fake://nowhere/',
+                                 database_connection=uuidsentinel.cell2)
+        cm.create()
 
-        event_type = mock_notification.call_args[1]['event_type']
-        priority = mock_notification.call_args[1]['priority']
-        publisher = mock_notification.call_args[1]['publisher']
-        payload = mock_notification.call_args[1]['payload']
+    def _create_services(self, *versions):
+        cells = objects.CellMappingList.get_all(self.context)
+        index = 0
+        for version in versions:
+            service = objects.Service(context=self.context,
+                                      binary='nova-compute')
+            service.version = version
+            cell = cells[index % len(cells)]
+            with context.target_cell(self.context, cell):
+                service.create()
+            index += 1
 
-        self.assertEqual(service_obj.host, publisher.host)
-        self.assertEqual(service_obj.binary, publisher.binary)
-        self.assertEqual(fields.NotificationPriority.INFO, priority)
-        self.assertEqual('service', event_type.object)
-        self.assertEqual(fields.NotificationAction.UPDATE,
-                         event_type.action)
-        for field in service.ServiceStatusPayload.SCHEMA:
-            if field in fake_service:
-                self.assertEqual(fake_service[field], getattr(payload, field))
+    @mock.patch('nova.objects.Service._send_notification')
+    @mock.patch('nova.objects.Service._check_minimum_version')
+    def test_version_all_cells(self, mock_check, mock_notify):
+        self._create_services(16, 16, 13, 16)
+        self.assertEqual(13, service.get_minimum_version_all_cells(
+            self.context, ['nova-compute']))
 
-        mock_notification.return_value.emit.assert_called_once_with(self.ctxt)
+    @mock.patch('nova.objects.service.LOG')
+    def test_get_minimum_version_checks_binary(self, mock_log):
+        ex = self.assertRaises(exception.ObjectActionError,
+                               service.get_minimum_version_all_cells,
+                               self.context, ['compute'])
+        self.assertIn('Invalid binary prefix', six.text_type(ex))
+        self.assertTrue(mock_log.warning.called)
 
-    @mock.patch('nova.db.service_update')
-    def test_service_update_with_notification(self, mock_db_service_update):
-        service_obj = objects.Service(context=self.ctxt, id=fake_service['id'])
-        mock_db_service_update.return_value = fake_service
-        for key, value in {'disabled': True,
-                           'disabled_reason': 'my reason',
-                           'forced_down': True}.items():
-            setattr(service_obj, key, value)
-            self._verify_notification(service_obj)
+    @mock.patch('nova.context.scatter_gather_all_cells')
+    def test_version_all_cells_with_fail(self, mock_scatter):
+        mock_scatter.return_value = {
+            'foo': {'nova-compute': 13},
+            'bar': exception.ServiceNotFound(service_id='fake'),
+        }
+        self.assertEqual(13, service.get_minimum_version_all_cells(
+            self.context, ['nova-compute']))
+        self.assertRaises(exception.CellTimeout,
+                          service.get_minimum_version_all_cells,
+                          self.context, ['nova-compute'],
+                          require_all=True)
 
-    @mock.patch('nova.objects.service.ServiceStatusNotification')
-    @mock.patch('nova.db.service_update')
-    def test_service_update_without_notification(self,
-                                                 mock_db_service_update,
-                                                 mock_notification):
-        service_obj = objects.Service(context=self.ctxt, id=fake_service['id'])
-        mock_db_service_update.return_value = fake_service
+    @mock.patch('nova.context.scatter_gather_all_cells')
+    def test_version_all_cells_with_timeout(self, mock_scatter):
+        mock_scatter.return_value = {
+            'foo': {'nova-compute': 13},
+            'bar': context.did_not_respond_sentinel,
+        }
+        self.assertEqual(13, service.get_minimum_version_all_cells(
+            self.context, ['nova-compute']))
+        self.assertRaises(exception.CellTimeout,
+                          service.get_minimum_version_all_cells,
+                          self.context, ['nova-compute'],
+                          require_all=True)
 
-        for key, value in {'report_count': 13,
-                           'last_seen_up': timeutils.utcnow()}.items():
-            setattr(service_obj, key, value)
-            service_obj.save()
-            self.assertFalse(mock_notification.called)
+    @mock.patch('nova.context.scatter_gather_all_cells')
+    def test_version_all_cells_exclude_zero_service(self, mock_scatter):
+        mock_scatter.return_value = {
+            'foo': {'nova-compute': 13},
+            'bar': {'nova-compute': 0},
+        }
+        self.assertEqual(13, service.get_minimum_version_all_cells(
+            self.context, ['nova-compute']))
