@@ -17,6 +17,7 @@ import copy
 
 import mock
 from oslo_serialization import jsonutils
+from oslo_utils.fixture import uuidsentinel
 
 import nova
 from nova.compute import vm_states
@@ -26,7 +27,6 @@ from nova.objects import fields
 from nova.pci import manager
 from nova import test
 from nova.tests.unit.pci import fakes as pci_fakes
-from nova.tests import uuidsentinel
 
 
 fake_pci = {
@@ -61,6 +61,7 @@ fake_db_dev = {
     'deleted_at': None,
     'deleted': None,
     'id': 1,
+    'uuid': uuidsentinel.pci_device1,
     'compute_node_id': 1,
     'address': '0000:00:00.1',
     'vendor_id': 'v',
@@ -76,21 +77,26 @@ fake_db_dev = {
     'parent_addr': None,
     }
 fake_db_dev_1 = dict(fake_db_dev, vendor_id='v1',
+                     uuid=uuidsentinel.pci_device1,
                      product_id='p1', id=2,
                      address='0000:00:00.2',
                      numa_node=0)
 fake_db_dev_2 = dict(fake_db_dev, id=3, address='0000:00:00.3',
+                     uuid=uuidsentinel.pci_device2,
                      numa_node=None, parent_addr='0000:00:00.1')
 fake_db_devs = [fake_db_dev, fake_db_dev_1, fake_db_dev_2]
 
 fake_db_dev_3 = dict(fake_db_dev, id=4, address='0000:00:01.1',
+                     uuid=uuidsentinel.pci_device3,
                      vendor_id='v2', product_id='p2',
                      numa_node=None, dev_type=fields.PciDeviceType.SRIOV_PF)
 fake_db_dev_4 = dict(fake_db_dev, id=5, address='0000:00:02.1',
+                     uuid=uuidsentinel.pci_device4,
                      numa_node=None, dev_type=fields.PciDeviceType.SRIOV_VF,
                      vendor_id='v2', product_id='p2',
                      parent_addr='0000:00:01.1')
 fake_db_dev_5 = dict(fake_db_dev, id=6, address='0000:00:02.2',
+                     uuid=uuidsentinel.pci_device5,
                      numa_node=None, dev_type=fields.PciDeviceType.SRIOV_VF,
                      vendor_id='v2', product_id='p2',
                      parent_addr='0000:00:01.1')
@@ -144,7 +150,7 @@ class PciDevTrackerTestCase(test.NoDBTestCase):
         super(PciDevTrackerTestCase, self).setUp()
         self.fake_context = context.get_admin_context()
         self.fake_devs = fake_db_devs[:]
-        self.stub_out('nova.db.pci_device_get_all_by_node',
+        self.stub_out('nova.db.api.pci_device_get_all_by_node',
             self._fake_get_pci_devices)
         # The fake_pci_whitelist must be called before creating the fake
         # devices
@@ -230,6 +236,40 @@ class PciDevTrackerTestCase(test.NoDBTestCase):
         tracker.update_devices_from_hypervisor_resources(fake_pci_devs_json)
         self.assertEqual(2, len(tracker.pci_devs))
 
+    @mock.patch("nova.pci.manager.LOG.debug")
+    def test_update_devices_from_hypervisor_resources_32bit_domain(
+            self, mock_debug):
+        self.flags(
+            group='pci',
+            passthrough_whitelist=[
+                '{"product_id":"2032", "vendor_id":"8086"}'])
+        # There are systems where 32 bit PCI domain is used. See bug 1897528
+        # for example. While nova (and qemu) does not support assigning such
+        # devices but the existence of such device in the system should not
+        # lead to an error.
+        fake_pci = {
+            'compute_node_id': 1,
+            'address': '10000:00:02.0',
+            'product_id': '2032',
+            'vendor_id': '8086',
+            'request_id': None,
+            'status': fields.PciDeviceStatus.AVAILABLE,
+            'dev_type': fields.PciDeviceType.STANDARD,
+            'parent_addr': None,
+            'numa_node': 0}
+
+        fake_pci_devs = [fake_pci]
+        fake_pci_devs_json = jsonutils.dumps(fake_pci_devs)
+        tracker = manager.PciDevTracker(self.fake_context)
+        # We expect that the device with 32bit PCI domain is ignored
+        tracker.update_devices_from_hypervisor_resources(fake_pci_devs_json)
+        self.assertEqual(0, len(tracker.pci_devs))
+        mock_debug.assert_called_once_with(
+            'Skipping PCI device %s reported by the hypervisor: %s',
+            {'address': '10000:00:02.0', 'parent_addr': None},
+            'The property domain (10000) is greater than the maximum '
+            'allowable value (FFFF).')
+
     def test_set_hvdev_new_dev(self):
         fake_pci_3 = dict(fake_pci, address='0000:00:00.4', vendor_id='v2')
         fake_pci_devs = [copy.deepcopy(fake_pci), copy.deepcopy(fake_pci_1),
@@ -306,6 +346,53 @@ class PciDevTrackerTestCase(test.NoDBTestCase):
         self.assertEqual(vfs, pf.child_devices)
         self.assertEqual(vfs[0].parent_device, pf)
 
+    def test_set_hvdev_remove_tree_maintained_with_allocations(self):
+        # Make sure the device tree is properly maintained when there are
+        # devices removed from the system that are allocated to vms.
+
+        all_devs = fake_db_devs_tree[:]
+        self._create_tracker(all_devs)
+        # we start with 3 devices
+        self.assertEqual(
+            3,
+            len([dev for dev in self.tracker.pci_devs
+                 if dev.status != fields.PciDeviceStatus.REMOVED]))
+        # we then allocate one device
+        pci_requests_obj = self._create_pci_requests_object(
+            [{'count': 1, 'spec': [{'vendor_id': 'v2'}]}])
+        # NOTE(sean-k-mooney): context, pci request, numa topology
+        claimed_dev = self.tracker.claim_instance(
+            mock.sentinel.context, pci_requests_obj, None)[0]
+
+        self.tracker._set_hvdevs(all_devs)
+        # and assert that no devices were removed
+        self.assertEqual(
+            0,
+            len([dev for dev in self.tracker.pci_devs
+                 if dev.status == fields.PciDeviceStatus.REMOVED]))
+        # we then try to remove the allocated device from the set reported
+        # by the driver.
+        fake_pci_devs = [dev for dev in all_devs
+                         if dev['address'] != claimed_dev.address]
+        with mock.patch("nova.pci.manager.LOG.warning") as log:
+            self.tracker._set_hvdevs(fake_pci_devs)
+            log.assert_called_once()
+            args = log.call_args_list[0][0]  # args of first call
+            self.assertIn('Unable to remove device with', args[0])
+        # and assert no devices are removed from the tracker
+        self.assertEqual(
+            0,
+            len([dev for dev in self.tracker.pci_devs
+                 if dev.status == fields.PciDeviceStatus.REMOVED]))
+        # free the device that was allocated and update tracker again
+        self.tracker._free_device(claimed_dev)
+        self.tracker._set_hvdevs(fake_pci_devs)
+        # and assert that one device is removed from the tracker
+        self.assertEqual(
+            1,
+            len([dev for dev in self.tracker.pci_devs
+                 if dev.status == fields.PciDeviceStatus.REMOVED]))
+
     def test_set_hvdev_changed_stal(self):
         pci_requests_obj = self._create_pci_requests_object(
             [{'count': 1, 'spec': [{'vendor_id': 'v1'}]}])
@@ -390,7 +477,7 @@ class PciDevTrackerTestCase(test.NoDBTestCase):
                        return_value=False)
     def test_save(self, migrate_mock):
         self.stub_out(
-                'nova.db.pci_device_update',
+                'nova.db.api.pci_device_update',
                 self._fake_pci_device_update)
         fake_pci_v3 = dict(fake_pci, address='0000:00:00.2', vendor_id='v3')
         fake_pci_devs = [copy.deepcopy(fake_pci), copy.deepcopy(fake_pci_2),
@@ -402,10 +489,10 @@ class PciDevTrackerTestCase(test.NoDBTestCase):
 
     def test_save_removed(self):
         self.stub_out(
-                'nova.db.pci_device_update',
+                'nova.db.api.pci_device_update',
                 self._fake_pci_device_update)
         self.stub_out(
-                'nova.db.pci_device_destroy',
+                'nova.db.api.pci_device_destroy',
                 self._fake_pci_device_destroy)
         self.destroy_called = 0
         self.assertEqual(len(self.tracker.pci_devs), 3)
@@ -472,6 +559,79 @@ class PciDevTrackerTestCase(test.NoDBTestCase):
         free_devs = self.tracker.pci_stats.get_free_devs()
         self.assertEqual(len(free_devs), 3)
 
+    def test_free_device(self):
+        pci_requests_obj = self._create_pci_requests_object(
+            [{'count': 1, 'spec': [{'vendor_id': 'v'}]}])
+        self.tracker.claim_instance(mock.sentinel.context,
+                                    pci_requests_obj, None)
+        self.tracker.update_pci_for_instance(None, self.inst, sign=1)
+        free_pci_device_ids = (
+            [dev.id for dev in self.tracker.pci_stats.get_free_devs()])
+        self.assertEqual(2, len(free_pci_device_ids))
+        allocated_devs = manager.get_instance_pci_devs(self.inst)
+        pci_device = allocated_devs[0]
+        self.assertNotIn(pci_device.id, free_pci_device_ids)
+        instance_uuid = self.inst['uuid']
+        self.assertIn(pci_device, self.tracker.allocations[instance_uuid])
+        self.tracker.free_device(pci_device, self.inst)
+        free_pci_device_ids = (
+            [dev.id for dev in self.tracker.pci_stats.get_free_devs()])
+        self.assertEqual(3, len(free_pci_device_ids))
+        self.assertIn(pci_device.id, free_pci_device_ids)
+        self.assertIsNone(self.tracker.allocations.get(instance_uuid))
+
+    def test_free_instance_claims(self):
+        # Create an InstancePCIRequest object
+        pci_requests_obj = self._create_pci_requests_object(
+            [{'count': 1, 'spec': [{'vendor_id': 'v'}]}])
+
+        # Claim a single PCI device
+        claimed_devs = self.tracker.claim_instance(mock.sentinel.context,
+                                                   pci_requests_obj, None)
+
+        # Assert we have exactly one claimed device for the given instance.
+        claimed_dev = claimed_devs[0]
+        instance_uuid = self.inst['uuid']
+        self.assertEqual(1, len(self.tracker.claims.get(instance_uuid)))
+        self.assertIn(claimed_dev.id,
+                      [pci_dev.id for pci_dev in
+                       self.tracker.claims.get(instance_uuid)])
+        self.assertIsNone(self.tracker.allocations.get(instance_uuid))
+
+        # Free instance claims
+        self.tracker.free_instance_claims(mock.sentinel.context, self.inst)
+
+        # Assert no claims for instance and all PCI devices are free
+        self.assertIsNone(self.tracker.claims.get(instance_uuid))
+        free_devs = self.tracker.pci_stats.get_free_devs()
+        self.assertEqual(len(fake_db_devs), len(free_devs))
+
+    def test_free_instance_allocations(self):
+        # Create an InstancePCIRequest object
+        pci_requests_obj = self._create_pci_requests_object(
+            [{'count': 1, 'spec': [{'vendor_id': 'v'}]}])
+        # Allocate a single PCI device
+        allocated_devs = self.tracker.claim_instance(mock.sentinel.context,
+                                                     pci_requests_obj, None)
+        self.tracker.allocate_instance(self.inst)
+
+        # Assert we have exactly one allocated device for the given instance.
+        allocated_dev = allocated_devs[0]
+        instance_uuid = self.inst['uuid']
+        self.assertIsNone(self.tracker.claims.get(instance_uuid))
+        self.assertEqual(1, len(self.tracker.allocations.get(instance_uuid)))
+        self.assertIn(allocated_dev.id,
+                      [pci_dev.id for pci_dev in
+                       self.tracker.allocations.get(instance_uuid)])
+
+        # Free instance allocations and assert claims did not change
+        self.tracker.free_instance_allocations(mock.sentinel.context,
+                                               self.inst)
+        # Assert all PCI devices are free.
+        self.assertIsNone(self.tracker.allocations.get(instance_uuid))
+        free_devs = self.tracker.pci_stats.get_free_devs()
+        self.assertEqual(len(fake_db_devs), len(free_devs))
+
 
 class PciGetInstanceDevs(test.NoDBTestCase):
 
@@ -488,3 +648,7 @@ class PciGetInstanceDevs(test.NoDBTestCase):
         self.load_attr_called = False
         manager.get_instance_pci_devs(objects.Instance())
         self.assertTrue(self.load_attr_called)
+
+    def test_get_devs_no_pci_devices(self):
+        inst = objects.Instance(pci_devices=None)
+        self.assertEqual([], manager.get_instance_pci_devs(inst))
