@@ -15,28 +15,27 @@
 """
 Tests For Scheduler Utils
 """
-import uuid
 
 import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 import six
 
 from nova.compute import flavors
 from nova.compute import utils as compute_utils
+from nova import context as nova_context
 from nova import exception
 from nova import objects
-from nova import rpc
 from nova.scheduler import utils as scheduler_utils
 from nova import test
 from nova.tests.unit import fake_instance
 from nova.tests.unit.objects import test_flavor
-from nova.tests import uuidsentinel as uuids
 
 
 class SchedulerUtilsTestCase(test.NoDBTestCase):
     """Test case for scheduler utils methods."""
     def setUp(self):
         super(SchedulerUtilsTestCase, self).setUp()
-        self.context = 'fake-context'
+        self.context = nova_context.get_context()
 
     def test_build_request_spec_without_image(self):
         instance = {'uuid': uuids.instance}
@@ -44,8 +43,7 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
 
         with mock.patch.object(flavors, 'extract_flavor') as mock_extract:
             mock_extract.return_value = instance_type
-            request_spec = scheduler_utils.build_request_spec(self.context,
-                                                              None,
+            request_spec = scheduler_utils.build_request_spec(None,
                                                               [instance])
             mock_extract.assert_called_once_with({'uuid': uuids.instance})
         self.assertEqual({}, request_spec['image'])
@@ -56,25 +54,26 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
 
         with mock.patch.object(instance, 'get_flavor') as mock_get:
             mock_get.return_value = instance_type
-            request_spec = scheduler_utils.build_request_spec(self.context,
-                                                              None,
+            request_spec = scheduler_utils.build_request_spec(None,
                                                               [instance])
             mock_get.assert_called_once_with()
         self.assertIsInstance(request_spec['instance_properties'], dict)
 
-    @mock.patch.object(rpc, 'get_notifier', return_value=mock.Mock())
+    @mock.patch('nova.compute.utils.notify_about_compute_task_error')
+    @mock.patch('nova.rpc.LegacyValidatingNotifier')
     @mock.patch.object(compute_utils, 'add_instance_fault_from_exc')
     @mock.patch.object(objects.Instance, 'save')
-    def test_set_vm_state_and_notify(self, mock_save, mock_add, mock_get):
+    def _test_set_vm_state_and_notify(self, mock_save, mock_add, mock_notifier,
+                                      mock_notify_task,
+                                      request_spec, payload_request_spec):
         expected_uuid = uuids.instance
-        request_spec = dict(instance_properties=dict(uuid='other-uuid'))
         updates = dict(vm_state='fake-vm-state')
         service = 'fake-service'
         method = 'fake-method'
         exc_info = 'exc_info'
 
-        payload = dict(request_spec=request_spec,
-                       instance_properties=request_spec.get(
+        payload = dict(request_spec=payload_request_spec,
+                       instance_properties=payload_request_spec.get(
                            'instance_properties', {}),
                        instance_id=expected_uuid,
                        state='fake-vm-state',
@@ -94,9 +93,41 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
                                          exc_info, mock.ANY)
         self.assertIsInstance(mock_add.call_args[0][1], objects.Instance)
         self.assertIsInstance(mock_add.call_args[0][3], tuple)
-        mock_get.return_value.error.assert_called_once_with(self.context,
-                                                            event_type,
-                                                            payload)
+        mock_notifier.return_value.error.assert_called_once_with(self.context,
+                                                                 event_type,
+                                                                 payload)
+        mock_notify_task.assert_called_once_with(
+            self.context, method, expected_uuid,
+            payload_request_spec, updates['vm_state'],
+            exc_info)
+
+    def test_set_vm_state_and_notify_request_spec_dict(self):
+        """Tests passing a legacy dict format request spec to
+        set_vm_state_and_notify.
+        """
+        request_spec = dict(instance_properties=dict(uuid=uuids.instance))
+        # The request_spec in the notification payload should be unchanged.
+        self._test_set_vm_state_and_notify(
+            request_spec=request_spec, payload_request_spec=request_spec)
+
+    def test_set_vm_state_and_notify_request_spec_object(self):
+        """Tests passing a RequestSpec object to set_vm_state_and_notify."""
+        request_spec = objects.RequestSpec.from_primitives(
+            self.context, dict(instance_properties=dict(uuid=uuids.instance)),
+            filter_properties=dict())
+        # The request_spec in the notification payload should be converted
+        # to the legacy format.
+        self._test_set_vm_state_and_notify(
+            request_spec=request_spec,
+            payload_request_spec=request_spec.to_legacy_request_spec_dict())
+
+    def test_set_vm_state_and_notify_request_spec_none(self):
+        """Tests passing None for the request_spec to set_vm_state_and_notify.
+        """
+        # The request_spec in the notification payload should be changed to
+        # just an empty dict.
+        self._test_set_vm_state_and_notify(
+            request_spec=None, payload_request_spec={})
 
     def test_build_filter_properties(self):
         sched_hints = {'hint': ['over-there']}
@@ -122,17 +153,18 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
         self.assertNotIn('forced_host', filt_props)
         self.assertNotIn('forced_node', filt_props)
 
-    def _test_populate_filter_props(self, host_state_obj=True,
+    def _test_populate_filter_props(self,
                                     with_retry=True,
                                     force_hosts=None,
-                                    force_nodes=None):
+                                    force_nodes=None,
+                                    no_limits=None):
         if force_hosts is None:
             force_hosts = []
         if force_nodes is None:
             force_nodes = []
         if with_retry:
-            if ((len(force_hosts) == 1 and len(force_nodes) <= 1)
-                 or (len(force_nodes) == 1 and len(force_hosts) <= 1)):
+            if ((len(force_hosts) == 1 and len(force_nodes) <= 1) or
+                 (len(force_nodes) == 1 and len(force_hosts) <= 1)):
                 filter_properties = dict(force_hosts=force_hosts,
                                          force_nodes=force_nodes)
             elif len(force_hosts) > 1 or len(force_nodes) > 1:
@@ -144,35 +176,37 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
         else:
             filter_properties = dict()
 
-        if host_state_obj:
-            class host_state(object):
-                host = 'fake-host'
-                nodename = 'fake-node'
-                limits = 'fake-limits'
+        if no_limits:
+            fake_limits = None
         else:
-            host_state = dict(host='fake-host',
-                              nodename='fake-node',
-                              limits='fake-limits')
+            fake_limits = objects.SchedulerLimits(vcpu=1, disk_gb=2,
+                    memory_mb=3, numa_topology=None)
+        selection = objects.Selection(service_host="fake-host",
+                nodename="fake-node", limits=fake_limits)
 
         scheduler_utils.populate_filter_properties(filter_properties,
-                                                   host_state)
+                                                   selection)
 
         enable_retry_force_hosts = not force_hosts or len(force_hosts) > 1
         enable_retry_force_nodes = not force_nodes or len(force_nodes) > 1
         if with_retry or enable_retry_force_hosts or enable_retry_force_nodes:
             # So we can check for 2 hosts
             scheduler_utils.populate_filter_properties(filter_properties,
-                                                       host_state)
+                                                       selection)
 
         if force_hosts:
             expected_limits = None
+        elif no_limits:
+            expected_limits = {}
+        elif isinstance(fake_limits, objects.SchedulerLimits):
+            expected_limits = fake_limits.to_dict()
         else:
-            expected_limits = 'fake-limits'
+            expected_limits = fake_limits
         self.assertEqual(expected_limits,
                          filter_properties.get('limits'))
 
-        if (with_retry and enable_retry_force_hosts
-                       and enable_retry_force_nodes):
+        if (with_retry and enable_retry_force_hosts and
+                enable_retry_force_nodes):
             self.assertEqual([['fake-host', 'fake-node'],
                               ['fake-host', 'fake-node']],
                              filter_properties['retry']['hosts'])
@@ -182,9 +216,6 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
     def test_populate_filter_props(self):
         self._test_populate_filter_props()
 
-    def test_populate_filter_props_host_dict(self):
-        self._test_populate_filter_props(host_state_obj=False)
-
     def test_populate_filter_props_no_retry(self):
         self._test_populate_filter_props(with_retry=False)
 
@@ -202,8 +233,11 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
         self._test_populate_filter_props(force_nodes=['force-node1',
                                                       'force-node2'])
 
+    def test_populate_filter_props_no_limits(self):
+        self._test_populate_filter_props(no_limits=True)
+
     def test_populate_retry_exception_at_max_attempts(self):
-        self.flags(scheduler_max_attempts=2)
+        self.flags(max_attempts=2, group='scheduler')
         msg = 'The exception text was preserved!'
         filter_properties = dict(retry=dict(num_attempts=2, hosts=[],
                                             exc_reason=[msg]))
@@ -243,15 +277,16 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
                                   [('bar', -2.1)])
 
     def test_validate_filters_configured(self):
-        self.flags(scheduler_default_filters='FakeFilter1,FakeFilter2')
+        self.flags(enabled_filters='FakeFilter1,FakeFilter2',
+                   group='filter_scheduler')
         self.assertTrue(scheduler_utils.validate_filter('FakeFilter1'))
         self.assertTrue(scheduler_utils.validate_filter('FakeFilter2'))
         self.assertFalse(scheduler_utils.validate_filter('FakeFilter3'))
 
     def test_validate_weighers_configured(self):
-        self.flags(scheduler_weight_classes=
-                   ['ServerGroupSoftAntiAffinityWeigher',
-                    'FakeFilter1'])
+        self.flags(weight_classes=[
+            'ServerGroupSoftAntiAffinityWeigher', 'FakeFilter1'],
+            group='filter_scheduler')
 
         self.assertTrue(scheduler_utils.validate_weigher(
             'ServerGroupSoftAntiAffinityWeigher'))
@@ -271,9 +306,9 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
 
         group = objects.InstanceGroup()
         group.name = 'pele'
-        group.uuid = str(uuid.uuid4())
+        group.uuid = uuids.fake
         group.members = [instance.uuid]
-        group.policies = [policy]
+        group.policy = policy
         return group
 
     def _get_group_details(self, group, policy=None):
@@ -290,7 +325,7 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
             group_info = scheduler_utils._get_group_details(
                 self.context, 'fake_uuid', group_hosts)
             self.assertEqual(
-                (set(['hostA', 'hostB']), [policy], group.members),
+                (set(['hostA', 'hostB']), policy, group.members),
                 group_info)
 
     def test_get_group_details(self):
@@ -304,16 +339,16 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
         self.assertIsNone(group_info)
 
     def _get_group_details_with_filter_not_configured(self, policy):
-        self.flags(scheduler_default_filters=['fake'])
-        self.flags(scheduler_weight_classes=['fake'])
+        self.flags(enabled_filters=['fake'], group='filter_scheduler')
+        self.flags(weight_classes=['fake'], group='filter_scheduler')
 
         instance = fake_instance.fake_instance_obj(self.context,
                 params={'host': 'hostA'})
 
         group = objects.InstanceGroup()
-        group.uuid = str(uuid.uuid4())
+        group.uuid = uuids.fake
         group.members = [instance.uuid]
-        group.policies = [policy]
+        group.policy = policy
 
         with test.nested(
             mock.patch.object(objects.InstanceGroup, 'get_by_instance_uuid',
@@ -334,43 +369,42 @@ class SchedulerUtilsTestCase(test.NoDBTestCase):
             self._get_group_details_with_filter_not_configured(policy)
 
     @mock.patch.object(scheduler_utils, '_get_group_details')
-    def test_setup_instance_group_in_filter_properties(self, mock_ggd):
+    def test_setup_instance_group_in_request_spec(self, mock_ggd):
         mock_ggd.return_value = scheduler_utils.GroupDetails(
-            hosts=set(['hostA', 'hostB']), policies=['policy'],
+            hosts=set(['hostA', 'hostB']), policy='policy',
             members=['instance1'])
-        spec = {'instance_properties': {'uuid': uuids.instance}}
-        filter_props = {'group_hosts': ['hostC']}
+        spec = objects.RequestSpec(instance_uuid=uuids.instance)
+        spec.instance_group = objects.InstanceGroup(hosts=['hostC'])
 
-        scheduler_utils.setup_instance_group(self.context, spec, filter_props)
+        scheduler_utils.setup_instance_group(self.context, spec)
 
         mock_ggd.assert_called_once_with(self.context, uuids.instance,
                                          ['hostC'])
-        expected_filter_props = {'group_updated': True,
-                                 'group_hosts': set(['hostA', 'hostB']),
-                                 'group_policies': ['policy'],
-                                 'group_members': ['instance1']}
-        self.assertEqual(expected_filter_props, filter_props)
+        # Given it returns a list from a set, make sure it's sorted.
+        self.assertEqual(['hostA', 'hostB'], sorted(spec.instance_group.hosts))
+        self.assertEqual('policy', spec.instance_group.policy)
+        self.assertEqual(['instance1'], spec.instance_group.members)
 
     @mock.patch.object(scheduler_utils, '_get_group_details')
     def test_setup_instance_group_with_no_group(self, mock_ggd):
         mock_ggd.return_value = None
-        spec = {'instance_properties': {'uuid': uuids.instance}}
-        filter_props = {'group_hosts': ['hostC']}
+        spec = objects.RequestSpec(instance_uuid=uuids.instance)
+        spec.instance_group = objects.InstanceGroup(hosts=['hostC'])
 
-        scheduler_utils.setup_instance_group(self.context, spec, filter_props)
+        scheduler_utils.setup_instance_group(self.context, spec)
 
         mock_ggd.assert_called_once_with(self.context, uuids.instance,
                                          ['hostC'])
-        self.assertNotIn('group_updated', filter_props)
-        self.assertNotIn('group_policies', filter_props)
-        self.assertEqual(['hostC'], filter_props['group_hosts'])
+        # Make sure the field isn't touched by the caller.
+        self.assertFalse(spec.instance_group.obj_attr_is_set('policies'))
+        self.assertEqual(['hostC'], spec.instance_group.hosts)
 
     @mock.patch.object(scheduler_utils, '_get_group_details')
     def test_setup_instance_group_with_filter_not_configured(self, mock_ggd):
         mock_ggd.side_effect = exception.NoValidHost(reason='whatever')
         spec = {'instance_properties': {'uuid': uuids.instance}}
-        filter_props = {'group_hosts': ['hostC']}
-
+        spec = objects.RequestSpec(instance_uuid=uuids.instance)
+        spec.instance_group = objects.InstanceGroup(hosts=['hostC'])
         self.assertRaises(exception.NoValidHost,
                           scheduler_utils.setup_instance_group,
-                          self.context, spec, filter_props)
+                          self.context, spec)
