@@ -16,12 +16,16 @@
 import abc
 import base64
 import contextlib
+import errno
 import functools
 import os
 import shutil
 
+from castellan import key_manager
+from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_service import loopingcall
 from oslo_utils import excutils
 from oslo_utils import fileutils
 from oslo_utils import strutils
@@ -31,9 +35,10 @@ import six
 import nova.conf
 from nova import exception
 from nova.i18n import _
-from nova.i18n import _LE, _LI, _LW
-from nova import image
-from nova import keymgr
+from nova.image import glance
+import nova.privsep.libvirt
+import nova.privsep.path
+from nova.storage import rbd_utils
 from nova import utils
 from nova.virt.disk import api as disk
 from nova.virt.image import model as imgmodel
@@ -41,17 +46,40 @@ from nova.virt import images
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt.storage import dmcrypt
 from nova.virt.libvirt.storage import lvm
-from nova.virt.libvirt.storage import rbd_utils
 from nova.virt.libvirt import utils as libvirt_utils
 
 CONF = nova.conf.CONF
 
 LOG = logging.getLogger(__name__)
-IMAGE_API = image.API()
+IMAGE_API = glance.API()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Image(object):
+# NOTE(neiljerram): Don't worry if this fails. This sometimes happens, with
+# EACCES (Permission Denied), when the base file is on an NFS client
+# filesystem. I don't understand why, but wonder if it's a similar problem as
+# the one that motivated using touch instead of utime in ec9d5e375e2.  In any
+# case, IIUC, timing isn't the primary thing that the image cache manager uses
+# to determine when the base file is in use. The primary mechanism for that is
+# whether there is a matching disk file for a current instance. The timestamp
+# on the base file is only used when deciding whether to delete a base file
+# that is _not_ in use; so it is not a big deal if that deletion happens
+# slightly earlier, for an unused base file, because of one of these preceding
+# utime calls having failed.
+# NOTE(mdbooth): Only use this method for updating the utime of an image cache
+# entry during disk creation.
+# TODO(mdbooth): Remove or rework this when we understand the problem.
+def _update_utime_ignore_eacces(path):
+    try:
+        nova.privsep.path.utime(path)
+    except OSError as e:
+        with excutils.save_and_reraise_exception(logger=LOG) as ctxt:
+            if e.errno == errno.EACCES:
+                LOG.warning("Ignoring failure to update utime of %(path)s: "
+                            "%(error)s", {'path': path, 'error': e})
+                ctxt.reraise = False
+
+
+class Image(metaclass=abc.ABCMeta):
 
     SUPPORTS_CLONE = False
 
@@ -65,9 +93,9 @@ class Image(object):
         """
         if (CONF.ephemeral_storage_encryption.enabled and
                 not self._supports_encryption()):
-            raise exception.NovaException(_('Incompatible settings: '
-                                  'ephemeral storage encryption is supported '
-                                  'only for LVM images.'))
+            msg = _('Incompatible settings: ephemeral storage encryption is '
+                    'supported only for LVM images.')
+            raise exception.InternalError(msg)
 
         self.path = path
 
@@ -117,23 +145,23 @@ class Image(object):
         """
         pass
 
-    def libvirt_info(self, disk_bus, disk_dev, device_type, cache_mode,
-                     extra_specs, hypervisor_version, boot_order=None):
+    def libvirt_info(self, disk_info, cache_mode,
+                     extra_specs, hypervisor_version, boot_order=None,
+                     disk_unit=None):
         """Get `LibvirtConfigGuestDisk` filled for this image.
 
-        :disk_dev: Disk bus device name
-        :disk_bus: Disk bus type
-        :device_type: Device type for this image.
+        :disk_info: Metadata generated by libvirt.blockinfo.get_disk_mapping
         :cache_mode: Caching mode for this image
         :extra_specs: Instance type extra specs dict.
         :hypervisor_version: the hypervisor version
         :boot_order: Disk device boot order
         """
+        disk_bus = disk_info['bus']
         info = vconfig.LibvirtConfigGuestDisk()
         info.source_type = self.source_type
-        info.source_device = device_type
+        info.source_device = disk_info['type']
         info.target_bus = disk_bus
-        info.target_dev = disk_dev
+        info.target_dev = disk_info['dev']
         info.driver_cache = cache_mode
         info.driver_discard = self.discard_mode
         info.driver_io = self.driver_io
@@ -144,15 +172,35 @@ class Image(object):
         info.source_path = self.path
         info.boot_order = boot_order
 
+        if disk_bus == 'scsi':
+            self.disk_scsi(info, disk_unit)
+
         self.disk_qos(info, extra_specs)
 
         return info
 
+    def disk_scsi(self, info, disk_unit):
+        # NOTE(melwitt): We set the device address unit number manually in the
+        # case of the virtio-scsi controller, in order to allow attachment of
+        # up to 256 devices. So, we should only be setting the address tag
+        # if we intend to set the unit number. Otherwise, we will let libvirt
+        # handle autogeneration of the address tag.
+        # See https://bugs.launchpad.net/nova/+bug/1792077 for details.
+        if disk_unit is not None:
+            # The driver is responsible to create the SCSI controller
+            # at index 0.
+            info.device_addr = vconfig.LibvirtConfigGuestDeviceAddressDrive()
+            info.device_addr.controller = 0
+            # In order to allow up to 256 disks handled by one
+            # virtio-scsi controller, the device addr should be
+            # specified.
+            info.device_addr.unit = disk_unit
+
     def disk_qos(self, info, extra_specs):
         tune_items = ['disk_read_bytes_sec', 'disk_read_iops_sec',
             'disk_write_bytes_sec', 'disk_write_iops_sec',
             'disk_total_bytes_sec', 'disk_total_iops_sec']
-        for key, value in six.iteritems(extra_specs):
+        for key, value in extra_specs.items():
             scope = key.split(':')
             if len(scope) > 1 and scope[0] == 'quota':
                 if scope[1] in tune_items:
@@ -200,19 +248,28 @@ class Image(object):
         :filename: Name of the file in the image directory
         :size: Size of created image in bytes (optional)
         """
-        @utils.synchronized(filename, external=True, lock_path=self.lock_path)
-        def fetch_func_sync(target, *args, **kwargs):
-            # The image may have been fetched while a subsequent
-            # call was waiting to obtain the lock.
-            if not os.path.exists(target):
-                fetch_func(target=target, *args, **kwargs)
-
         base_dir = os.path.join(CONF.instances_path,
-                                CONF.image_cache_subdirectory_name)
+                                CONF.image_cache.subdirectory_name)
         if not os.path.exists(base_dir):
             fileutils.ensure_tree(base_dir)
         base = os.path.join(base_dir, filename)
 
+        @utils.synchronized(filename, external=True, lock_path=self.lock_path)
+        def fetch_func_sync(target, *args, **kwargs):
+            # NOTE(mdbooth): This method is called as a callback by the
+            # create_image() method of a specific backend. It assumes that
+            # target will be in the image cache, which is why it holds a
+            # lock, and does not overwrite an existing file. However,
+            # this is not true for all backends. Specifically Lvm writes
+            # directly to the target rather than to the image cache,
+            # and additionally it creates the target in advance.
+            # This guard is only relevant in the context of the lock if the
+            # target is in the image cache. If it isn't, we should
+            # call fetch_func. The lock we're holding is also unnecessary in
+            # that case, but it will not result in incorrect behaviour.
+            if target != base or not os.path.exists(target):
+                fetch_func(target=target, *args, **kwargs)
+
         if not self.exists() or not os.path.exists(base):
             self.create_image(fetch_func_sync, base, size,
                               *args, **kwargs)
@@ -225,7 +282,7 @@ class Image(object):
 
             if (self.preallocate and self._can_fallocate() and
                     os.access(self.path, os.W_OK)):
-                utils.execute('fallocate', '-n', '-l', size, self.path)
+                processutils.execute('fallocate', '-n', '-l', size, self.path)
 
     def _can_fallocate(self):
         """Check once per class, whether fallocate(1) is available,
@@ -234,13 +291,13 @@ class Image(object):
         can_fallocate = getattr(self.__class__, 'can_fallocate', None)
         if can_fallocate is None:
             test_path = self.path + '.fallocate_test'
-            _out, err = utils.trycmd('fallocate', '-l', '1', test_path)
+            _out, err = processutils.trycmd('fallocate', '-l', '1', test_path)
             fileutils.delete_if_exists(test_path)
             can_fallocate = not err
             self.__class__.can_fallocate = can_fallocate
             if not can_fallocate:
-                LOG.warning(_LW('Unable to preallocate image at path: '
-                                '%(path)s'), {'path': self.path})
+                LOG.warning('Unable to preallocate image at path: %(path)s',
+                            {'path': self.path})
         return can_fallocate
 
     def verify_base_size(self, base, size, base_size=0):
@@ -265,18 +322,27 @@ class Image(object):
             base_size = self.get_disk_size(base)
 
         if size < base_size:
-            msg = _LE('%(base)s virtual size %(base_size)s '
-                      'larger than flavor root disk size %(size)s')
-            LOG.error(msg % {'base': base,
-                              'base_size': base_size,
-                              'size': size})
+            LOG.error('%(base)s virtual size %(base_size)s '
+                      'larger than flavor root disk size %(size)s',
+                      {'base': base,
+                       'base_size': base_size,
+                       'size': size})
             raise exception.FlavorDiskSmallerThanImage(
                 flavor_size=size, image_size=base_size)
 
     def get_disk_size(self, name):
         return disk.get_disk_size(name)
 
+    @abc.abstractmethod
     def snapshot_extract(self, target, out_format):
+        """Extract a snapshot of the image.
+
+        This is used during cold (offline) snapshots. Live snapshots
+        while the guest is still running are handled separately.
+
+        :param target: The target path for the image snapshot.
+        :param out_format: The image snapshot format.
+        """
         raise NotImplementedError()
 
     def _get_driver_format(self):
@@ -328,7 +394,7 @@ class Image(object):
                 with open(self.disk_info_path) as disk_info_file:
                     line = disk_info_file.read().rstrip()
                     dct = _dict_from_line(line)
-                    for path, driver_format in six.iteritems(dct):
+                    for path, driver_format in dct.items():
                         if path == self.path:
                             return driver_format
             driver_format = self._get_driver_format()
@@ -364,9 +430,20 @@ class Image(object):
         raise exception.ImageUnacceptable(image_id=image_id_or_uri,
                                           reason=reason)
 
+    def flatten(self):
+        """Flatten an image.
+
+        The implementation of this method is optional and therefore is
+        not an abstractmethod.
+        """
+        raise NotImplementedError('flatten() is not implemented')
+
     def direct_snapshot(self, context, snapshot_name, image_format, image_id,
                         base_image_id):
-        """Prepare a snapshot for direct reference from glance
+        """Prepare a snapshot for direct reference from glance.
+
+        The implementation of this method is optional and therefore is
+        not an abstractmethod.
 
         :raises: exception.ImageUnacceptable if it cannot be
                  referenced directly in the specified image format
@@ -387,6 +464,7 @@ class Image(object):
         """Get an image's name of a base file."""
         return os.path.split(base)[-1]
 
+    @abc.abstractmethod
     def get_model(self, connection):
         """Get the image information model
 
@@ -461,10 +539,9 @@ class Flat(Image):
             data = images.qemu_img_info(self.path)
             return data.file_format
         except exception.InvalidDiskInfo as e:
-            LOG.info(_LI('Failed to get image info from path %(path)s; '
-                         'error: %(error)s'),
-                      {'path': self.path,
-                       'error': e})
+            LOG.info('Failed to get image info from path %(path)s; '
+                     'error: %(error)s',
+                     {'path': self.path, 'error': e})
             return 'raw'
 
     def _supports_encryption(self):
@@ -505,7 +582,7 @@ class Flat(Image):
 
             # NOTE(mikal): Update the mtime of the base file so the image
             # cache manager knows it is in use.
-            libvirt_utils.update_mtime(base)
+            _update_utime_ignore_eacces(base)
             self.verify_base_size(base, size)
             if not os.path.exists(self.path):
                 with fileutils.remove_path_on_error(self.path):
@@ -561,7 +638,7 @@ class Qcow2(Image):
 
         # NOTE(ankit): Update the mtime of the base file so the image
         # cache manager knows it is in use.
-        libvirt_utils.update_mtime(base)
+        _update_utime_ignore_eacces(base)
         self.verify_base_size(base, size)
 
         legacy_backing_size = None
@@ -621,7 +698,7 @@ class Lvm(Image):
         self.ephemeral_key_uuid = instance.get('ephemeral_key_uuid')
 
         if self.ephemeral_key_uuid is not None:
-            self.key_manager = keymgr.API(CONF)
+            self.key_manager = key_manager.API(CONF)
         else:
             self.key_manager = None
 
@@ -648,8 +725,8 @@ class Lvm(Image):
 
         super(Lvm, self).__init__(path, "block", "raw", is_block_dev=True)
 
-        # TODO(pbrady): possibly deprecate libvirt.sparse_logical_volumes
-        # for the more general preallocate_images
+        # TODO(sbauza): Remove the config option usage and default the
+        # LVM logical volume creation to preallocate the full size only.
         self.sparse = CONF.libvirt.sparse_logical_volumes
         self.preallocate = not self.sparse
 
@@ -676,7 +753,7 @@ class Lvm(Image):
         def create_lvm_image(base, size):
             base_size = disk.get_disk_size(base)
             self.verify_base_size(base, size, base_size=base_size)
-            resize = size > base_size
+            resize = size > base_size if size else False
             size = size if resize else base_size
             lvm.create_volume(self.vg, self.lv,
                                          size, sparse=self.sparse)
@@ -706,10 +783,10 @@ class Lvm(Image):
                             self.ephemeral_key_uuid).get_encoded()
                 except Exception:
                     with excutils.save_and_reraise_exception():
-                        LOG.error(_LE("Failed to retrieve ephemeral encryption"
-                                      " key"))
+                        LOG.error("Failed to retrieve ephemeral "
+                                  "encryption key")
             else:
-                raise exception.NovaException(
+                raise exception.InternalError(
                     _("Instance disk to be encrypted but no context provided"))
         # Generate images with specified size right on volume
         if generated and size:
@@ -754,7 +831,7 @@ class Rbd(Image):
 
     SUPPORTS_CLONE = True
 
-    def __init__(self, instance=None, disk_name=None, path=None, **kwargs):
+    def __init__(self, instance=None, disk_name=None, path=None):
         if not CONF.libvirt.images_rbd_pool:
             raise RuntimeError(_('You should specify'
                                  ' images_rbd_pool'
@@ -768,62 +845,58 @@ class Rbd(Image):
         else:
             self.rbd_name = '%s_%s' % (instance.uuid, disk_name)
 
-        self.pool = CONF.libvirt.images_rbd_pool
-        self.rbd_user = CONF.libvirt.rbd_user
-        self.ceph_conf = CONF.libvirt.images_rbd_ceph_conf
+        self.driver = rbd_utils.RBDDriver()
 
-        path = 'rbd:%s/%s' % (self.pool, self.rbd_name)
-        if self.rbd_user:
-            path += ':id=' + self.rbd_user
-        if self.ceph_conf:
-            path += ':conf=' + self.ceph_conf
+        path = 'rbd:%s/%s' % (self.driver.pool, self.rbd_name)
+        if self.driver.rbd_user:
+            path += ':id=' + self.driver.rbd_user
+        if self.driver.ceph_conf:
+            path += ':conf=' + self.driver.ceph_conf
 
         super(Rbd, self).__init__(path, "block", "rbd", is_block_dev=False)
 
-        self.driver = rbd_utils.RBDDriver(
-            pool=self.pool,
-            ceph_conf=self.ceph_conf,
-            rbd_user=self.rbd_user)
-
         self.discard_mode = CONF.libvirt.hw_disk_discard
 
-    def libvirt_info(self, disk_bus, disk_dev, device_type, cache_mode,
-            extra_specs, hypervisor_version, boot_order=None):
+    def libvirt_info(self, disk_info, cache_mode,
+                     extra_specs, hypervisor_version, boot_order=None,
+                     disk_unit=None):
         """Get `LibvirtConfigGuestDisk` filled for this image.
 
-        :disk_dev: Disk bus device name
-        :disk_bus: Disk bus type
-        :device_type: Device type for this image.
+        :disk_info: Metadata generated by libvirt.blockinfo.get_disk_mapping
         :cache_mode: Caching mode for this image
         :extra_specs: Instance type extra specs dict.
         :hypervisor_version: the hypervisor version
         :boot_order: Disk device boot order
         """
         info = vconfig.LibvirtConfigGuestDisk()
+        disk_bus = disk_info['bus']
 
         hosts, ports = self.driver.get_mon_addrs()
-        info.source_device = device_type
+        info.source_device = disk_info['type']
         info.driver_format = 'raw'
         info.driver_cache = cache_mode
         info.driver_discard = self.discard_mode
         info.target_bus = disk_bus
-        info.target_dev = disk_dev
+        info.target_dev = disk_info['dev']
         info.source_type = 'network'
         info.source_protocol = 'rbd'
-        info.source_name = '%s/%s' % (self.pool, self.rbd_name)
+        info.source_name = '%s/%s' % (self.driver.pool, self.rbd_name)
         info.source_hosts = hosts
         info.source_ports = ports
         info.boot_order = boot_order
-        auth_enabled = (CONF.libvirt.rbd_user is not None)
+        auth_enabled = (self.driver.rbd_user is not None)
         if CONF.libvirt.rbd_secret_uuid:
             info.auth_secret_uuid = CONF.libvirt.rbd_secret_uuid
             auth_enabled = True  # Force authentication locally
-            if CONF.libvirt.rbd_user:
-                info.auth_username = CONF.libvirt.rbd_user
+            if self.driver.rbd_user:
+                info.auth_username = self.driver.rbd_user
         if auth_enabled:
             info.auth_secret_type = 'ceph'
             info.auth_secret_uuid = CONF.libvirt.rbd_secret_uuid
 
+        if disk_bus == 'scsi':
+            self.disk_scsi(info, disk_unit)
+
         self.disk_qos(info, extra_specs)
 
         return info
@@ -842,9 +915,28 @@ class Rbd(Image):
         """
         return self.driver.size(self.rbd_name)
 
+    @staticmethod
+    def _remove_non_raw_cache_image(base):
+        # NOTE(boxiang): If the cache image file exists, we will check
+        # the format of it. Only raw format image is compatible for
+        # RBD image backend. If format is not raw, we will remove it
+        # at first. We limit force_raw_images to True this time. So
+        # the format of new cache image must be raw.
+        # We can remove this in 'U' version later.
+        if not os.path.exists(base):
+            return True
+        image_format = images.qemu_img_info(base)
+        if image_format.file_format != 'raw':
+            try:
+                os.remove(base)
+            except OSError as e:
+                LOG.warning("Ignoring failure to remove %(path)s: "
+                            "%(error)s", {'path': base, 'error': e})
+
     def create_image(self, prepare_template, base, size, *args, **kwargs):
 
         if not self.exists():
+            self._remove_non_raw_cache_image(base)
             prepare_template(target=base, *args, **kwargs)
 
         # prepare_template() may have cloned the image into a new rbd
@@ -866,12 +958,82 @@ class Rbd(Image):
     def is_shared_block_storage():
         return True
 
-    def clone(self, context, image_id_or_uri):
+    def copy_to_store(self, context, image_meta):
+        store_name = CONF.libvirt.images_rbd_glance_store_name
+        image_id = image_meta['id']
+        try:
+            IMAGE_API.copy_image_to_store(context, image_id, store_name)
+        except exception.ImageBadRequest:
+            # NOTE(danms): This means that we raced with another node to start
+            # the copy. Fall through to polling the image for completion
+            pass
+        except exception.ImageImportImpossible as exc:
+            # NOTE(danms): This means we can not do this operation at all,
+            # so fold this into the kind of imagebackend failure that is
+            # expected by our callers
+            raise exception.ImageUnacceptable(image_id=image_id,
+                                              reason=str(exc))
+
+        def _wait_for_copy():
+            image = IMAGE_API.get(context, image_id, include_locations=True)
+            if store_name in image.get('os_glance_failed_import', []):
+                # Our store is reported as failed
+                raise loopingcall.LoopingCallDone('failed import')
+            elif (store_name not in image.get('os_glance_importing_to_stores',
+                                              []) and
+                  store_name in image['stores']):
+                # No longer importing and our store is listed in the stores
+                raise loopingcall.LoopingCallDone()
+            else:
+                LOG.debug('Glance reports copy of image %(image)s to '
+                          'rbd store %(store)s is still in progress',
+                          {'image': image_id,
+                           'store': store_name})
+                return True
+
+        LOG.info('Asking glance to copy image %(image)s to our '
+                 'rbd store %(store)s',
+                 {'image': image_id,
+                  'store': store_name})
+
+        timer = loopingcall.FixedIntervalWithTimeoutLoopingCall(_wait_for_copy)
+
+        # NOTE(danms): We *could* do something more complicated like try
+        # to scale our polling interval based on image size. The problem with
+        # that is that we do not get progress indication from Glance, so if
+        # we scale our interval to something long, and happen to poll right
+        # near the end of the copy, we will wait another long interval before
+        # realizing that the copy is complete. A simple interval per compute
+        # allows an operator to set this short on central/fast/inexpensive
+        # computes, and longer on nodes that are remote/slow/expensive across
+        # a slower link.
+        interval = CONF.libvirt.images_rbd_glance_copy_poll_interval
+        timeout = CONF.libvirt.images_rbd_glance_copy_timeout
+        try:
+            result = timer.start(interval=interval, timeout=timeout).wait()
+        except loopingcall.LoopingCallTimeOut:
+            raise exception.ImageUnacceptable(
+                image_id=image_id,
+                reason='Copy to store %(store)s timed out' % {
+                    'store': store_name})
+
+        if result is not True:
+            raise exception.ImageUnacceptable(
+                image_id=image_id,
+                reason=('Copy to store %(store)s unsuccessful '
+                        'because: %(reason)s') % {'store': store_name,
+                                                  'reason': result})
+
+        LOG.info('Image %(image)s copied to rbd store %(store)s',
+                 {'image': image_id,
+                  'store': store_name})
+
+    def clone(self, context, image_id_or_uri, copy_to_store=True):
         image_meta = IMAGE_API.get(context, image_id_or_uri,
                                    include_locations=True)
         locations = image_meta['locations']
 
-        LOG.debug('Image locations are: %(locs)s' % {'locs': locations})
+        LOG.debug('Image locations are: %(locs)s', {'locs': locations})
 
         if image_meta.get('disk_format') not in ['raw', 'iso']:
             reason = _('Image is not raw format')
@@ -883,10 +1045,29 @@ class Rbd(Image):
                 LOG.debug('Selected location: %(loc)s', {'loc': location})
                 return self.driver.clone(location, self.rbd_name)
 
+        # Not clone-able in our ceph, so try to get glance to copy it for us
+        # and then retry
+        if CONF.libvirt.images_rbd_glance_store_name and copy_to_store:
+            self.copy_to_store(context, image_meta)
+            return self.clone(context, image_id_or_uri, copy_to_store=False)
+
         reason = _('No image locations are accessible')
         raise exception.ImageUnacceptable(image_id=image_id_or_uri,
                                           reason=reason)
 
+    def flatten(self):
+        # NOTE(vdrok): only flatten images if they are not already flattened,
+        # meaning that parent info is present
+        try:
+            self.driver.parent_info(self.rbd_name, pool=self.driver.pool)
+        except exception.ImageUnacceptable:
+            LOG.debug(
+                "Image %(img)s from pool %(pool)s has no parent info, "
+                "consider it already flat", {
+                    'img': self.rbd_name, 'pool': self.driver.pool})
+        else:
+            self.driver.flatten(self.rbd_name, pool=self.driver.pool)
+
     def get_model(self, connection):
         secret = None
         if CONF.libvirt.rbd_secret_uuid:
@@ -894,12 +1075,15 @@ class Rbd(Image):
                 CONF.libvirt.rbd_secret_uuid)
             secret = base64.b64encode(secretobj.value())
 
-        hosts, ports = self.driver.get_mon_addrs()
+        # Brackets are stripped from IPv6 addresses normally for libvirt XML,
+        # but the servers list is for libguestfs, which needs the brackets
+        # so the joined string is similar to '[::1]:6789'
+        hosts, ports = self.driver.get_mon_addrs(strip_brackets=False)
         servers = [str(':'.join(k)) for k in zip(hosts, ports)]
 
         return imgmodel.RBDImage(self.rbd_name,
-                                 self.pool,
-                                 self.rbd_user,
+                                 self.driver.pool,
+                                 self.driver.rbd_user,
                                  secret,
                                  servers)
 
@@ -973,7 +1157,7 @@ class Rbd(Image):
         self.driver.create_snap(self.rbd_name, snapshot_name, protect=True)
         location = {'url': 'rbd://%(fsid)s/%(pool)s/%(image)s/%(snap)s' %
                            dict(fsid=fsid,
-                                pool=self.pool,
+                                pool=self.driver.pool,
                                 image=self.rbd_name,
                                 snap=snapshot_name)}
         try:
@@ -1017,19 +1201,48 @@ class Ploop(Image):
 
         self.resolve_driver_format()
 
+    # Create new ploop disk (in case of epehemeral) or
+    # copy ploop disk from glance image
     def create_image(self, prepare_template, base, size, *args, **kwargs):
-        filename = os.path.split(base)[-1]
+        filename = os.path.basename(base)
 
+        # Copy main file of ploop disk, restore DiskDescriptor.xml for it
+        # and resize if necessary
         @utils.synchronized(filename, external=True, lock_path=self.lock_path)
-        def create_ploop_image(base, target, size):
+        def _copy_ploop_image(base, target, size):
+            # Ploop disk is a directory with data file(root.hds) and
+            # DiskDescriptor.xml, so create this dir
+            fileutils.ensure_tree(target)
             image_path = os.path.join(target, "root.hds")
             libvirt_utils.copy_image(base, image_path)
-            utils.execute('ploop', 'restore-descriptor', '-f', self.pcs_format,
-                          target, image_path)
+            nova.privsep.libvirt.ploop_restore_descriptor(target,
+                                                          image_path,
+                                                          self.pcs_format)
             if size:
                 self.resize_image(size)
 
-        if not os.path.exists(self.path):
+        # Generating means that we create empty ploop disk
+        generating = 'image_id' not in kwargs
+        remove_func = functools.partial(fileutils.delete_if_exists,
+                                        remove=shutil.rmtree)
+        if generating:
+            if os.path.exists(self.path):
+                return
+            with fileutils.remove_path_on_error(self.path, remove=remove_func):
+                prepare_template(target=self.path, *args, **kwargs)
+        else:
+            # Create ploop disk from glance image
+            if not os.path.exists(base):
+                prepare_template(target=base, *args, **kwargs)
+            else:
+                # Disk already exists in cache, just update time
+                _update_utime_ignore_eacces(base)
+            self.verify_base_size(base, size)
+
+            if os.path.exists(self.path):
+                return
+
+            # Get format for ploop disk
             if CONF.force_raw_images:
                 self.pcs_format = "raw"
             else:
@@ -1041,27 +1254,16 @@ class Ploop(Image):
                 elif format == "raw":
                     self.pcs_format = "raw"
                 else:
-                    reason = _("PCS doesn't support images in %s format."
-                                " You should either set force_raw_images=True"
-                                " in config or upload an image in ploop"
-                                " or raw format.") % format
+                    reason = _("Ploop image backend doesn't support images in"
+                               " %s format. You should either set"
+                               " force_raw_images=True in config or upload an"
+                               " image in ploop or raw format.") % format
                     raise exception.ImageUnacceptable(
                                         image_id=kwargs["image_id"],
                                         reason=reason)
 
-        if not os.path.exists(base):
-            prepare_template(target=base, *args, **kwargs)
-        self.verify_base_size(base, size)
-
-        if os.path.exists(self.path):
-            return
-
-        fileutils.ensure_tree(self.path)
-
-        remove_func = functools.partial(fileutils.delete_if_exists,
-                                        remove=shutil.rmtree)
-        with fileutils.remove_path_on_error(self.path, remove=remove_func):
-            create_ploop_image(base, self.path, size)
+            with fileutils.remove_path_on_error(self.path, remove=remove_func):
+                _copy_ploop_image(base, self.path, size)
 
     def resize_image(self, size):
         image = imgmodel.LocalFileImage(self.path, imgmodel.FORMAT_PLOOP)
@@ -1074,6 +1276,9 @@ class Ploop(Image):
                                        target,
                                        out_format)
 
+    def get_model(self, connection):
+        return imgmodel.LocalFileImage(self.path, imgmodel.FORMAT_PLOOP)
+
 
 class Backend(object):
     def __init__(self, use_cow):
@@ -1095,22 +1300,28 @@ class Backend(object):
             raise RuntimeError(_('Unknown image_type=%s') % image_type)
         return image
 
-    def image(self, instance, disk_name, image_type=None):
-        """Constructs image for selected backend
+    def by_name(self, instance, name, image_type=None):
+        """Return an Image object for a disk with the given name.
 
-        :instance: Instance name.
-        :name: Image name.
-        :image_type: Image type.
-                     Optional, is CONF.libvirt.images_type by default.
+        :param instance: the instance which owns this disk
+        :param name: The name of the disk
+        :param image_type: (Optional) Image type.
+                           Default is CONF.libvirt.images_type.
+        :return: An Image object for the disk with given name and instance.
+        :rtype: Image
         """
         backend = self.backend(image_type)
-        return backend(instance=instance, disk_name=disk_name)
+        return backend(instance=instance, disk_name=name)
 
-    def snapshot(self, instance, disk_path, image_type=None):
-        """Returns snapshot for given image
+    def by_libvirt_path(self, instance, path, image_type=None):
+        """Return an Image object for a disk with the given libvirt path.
 
-        :path: path to image
-        :image_type: type of image
+        :param instance: The instance which owns this disk.
+        :param path: The libvirt representation of the image's path.
+        :param image_type: (Optional) Image type.
+                           Default is CONF.libvirt.images_type.
+        :return: An Image object for the given libvirt path.
+        :rtype: Image
         """
         backend = self.backend(image_type)
-        return backend(instance=instance, path=disk_path)
+        return backend(instance=instance, path=path)
