@@ -344,14 +344,21 @@ def _update_vif_xml(xml_doc, migrate_data, get_vif_config):
     instance_uuid = xml_doc.findtext('uuid')
     parser = etree.XMLParser(remove_blank_text=True)
     interface_nodes = xml_doc.findall('./devices/interface')
-    migrate_vif_by_mac = {vif.source_vif['address']: vif
+    # MAC address stored for port in neutron DB and in domain XML
+    # might be in different cases, so to harmonize that
+    # we convert MAC to lower case for dict key.
+    migrate_vif_by_mac = {vif.source_vif['address'].lower(): vif
                           for vif in migrate_data.vifs}
     for interface_dev in interface_nodes:
         mac = interface_dev.find('mac')
         mac = mac if mac is not None else {}
         mac_addr = mac.get('address')
         if mac_addr:
-            migrate_vif = migrate_vif_by_mac[mac_addr]
+            # MAC address stored in libvirt should always be normalized
+            # and stored in lower case. But just to be extra safe here
+            # we still normalize MAC retrieved from XML to be absolutely
+            # sure it will be the same with the Neutron provided one.
+            migrate_vif = migrate_vif_by_mac[mac_addr.lower()]
             vif = migrate_vif.get_dest_vif()
             # get_vif_config is a partial function of
             # nova.virt.libvirt.vif.LibvirtGenericVIFDriver.get_config
